/*>>keyflag<< "%n"***********************************************************
    "SMPLFILE.C"

    Copyright (c) 1993-95  Pixel Translations, Incorporated.
    All Rights Reserved.
    Confidential and Proprietary to Pixel Translations, Incorporated.

	Much of the file I/O part of smplview
*****************************************************************************
  TLIB Keywords: %v=version, %d=file date, %f=file date and time, %t=time
 		 %w=who last, %l=lock status, %n=file name
*****************************************************************************/


/*>>keyflag<< * %n %f %l **/
/* SMPLFILE.C 11-Jul-96,14:16:16  */

/*>>keyflag<< "&(#)%n v%v %f" */
static char szVersionId[] = "&(#)SMPLFILE.C v23 11-Jul-96,14:16:16";

#ifndef WINDOWS
#define WINDOWS
#endif
#include <pixdflt.h>
#include <pixm.h>
#include <pixif.h>
#include "smplview.h"
#include <autopipe.h>
#include <commdlg.h>
#include "smplfile.h"
#include <pixsvdlg.h>

#if (defined INCLUDE_NOTES) && (INCLUDE_NOTES == 0)
#undef INCLUDE_NOTES
#endif

#ifdef INCLUDE_NOTES
#include <pixnote.h>
#define MERGE_NOTES
#endif

#ifdef INCLUDE_NOTES
/* notes.c functions */
extern int ReadAnnotations(char FAR *FName, PIXM_PAGE hPage);
extern int SaveAnnotations(char FAR *FName);
extern WORD NoteHndl[];
#endif

extern PixDrvHandle hPixIF;
extern PixDrvHandle hPixmIsis;
extern PixDrvHandle hFromPixm;
extern PixDrvHandle hPixfPack;
extern char buffer[8192];
extern PIXM_SETTINGS pSettings;

#define ISIS

int FindTiffInList(char FAR *lpszFormats)
{
	int ans = 1;
	while (*lpszFormats) {
		if (!PixMemStrncmp(lpszFormats, "TIFF", 4))
			return ans;
		else {
			/* skip format */
			lpszFormats += PixMemStrlen(lpszFormats) + 1;
			/* skip extension */
			lpszFormats += PixMemStrlen(lpszFormats) + 1;
		}
		ans++;
	}
	/* couldn't find TIFF */
	return 1;
}

int FileToPixm(HWND hWnd, PIXM_PAGE FAR *lpPage)
{
	register int r;
    OPENFILENAME ofn;
    char szFile[PIX_FILE_MAXLEN];
	INT32 lValue, lLen;
	HCURSOR hOldCursor;
	char FAR *lpszFormats = 0;
	BOOL bFreeFormats = 0;

	/* turn on Known Formats and All Files options */
	PixTagSetLong(&hPixIF, TAG_KNOWNFMTSFLAG, 0, 1L);
	PixTagSetLong(&hPixIF, TAG_ALLFILESFLAG, 0, 1L);

	/* find out how long Formats string is */
	if (PixTagGetLength(&hPixIF, TAG_AVAILFMTS, &lLen))
		lLen = 0;

	if (lLen) {
		/* allocate string to put formats in */
		lpszFormats = PixMemAllocPtr((int)lLen, PIX_MEM_ZERO);
		if (lpszFormats) {
			/* get string */
			if (PixTagGetAscii(&hPixIF, TAG_AVAILFMTS, 0, lpszFormats)) {
				/* free if get ascii fails */
				PixMemFreePtr(lpszFormats);
				lpszFormats = 0;
			} else {
				bFreeFormats = TRUE;
			}
		}
	}
	if (!lpszFormats)
		lpszFormats = "TIFF (*.tif)\0*.tif\0\0";

    szFile[0] = 0;
    memset(&ofn, 0, sizeof(OPENFILENAME));
    ofn.lStructSize = sizeof(OPENFILENAME);
    ofn.hwndOwner = hWnd;
    ofn.lpstrFilter = lpszFormats;
    ofn.nFilterIndex = FindTiffInList(lpszFormats);
    ofn.lpstrFile = szFile;
    ofn.nMaxFile = sizeof(szFile);
    ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

    if (!GetOpenFileName(&ofn))
    	return PIX_ERR_CANCEL;

	if (bFreeFormats) {
		PixMemFreePtr(lpszFormats);
		bFreeFormats = 0;
	}
	lpszFormats = 0;

	/* Initialize F_PIXIF with the file name */
	lValue = PixTagSetAscii(&hPixIF, TAG_INPUTNAME, szFile);
	if (r = Check(&hPixIF, lValue)) {
		return r;
	}

    if (r = Check(0, PixmPageNew(lpPage)))
    	return r;
    if (r = Check(&hPixmIsis,
			PixDrvInitialize(&hPixmIsis, 0, (INT32)(void FAR *)*lpPage))) {
        PixmPageFree(*lpPage);
        return r;
    }

    PixDrvLink(&hPixIF, &hPixmIsis);
    hOldCursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
    lValue = PixRunZone(&hPixIF, buffer, sizeof buffer);
    SetCursor(hOldCursor);

	if (lValue < 0) {
        PixmPageFree(*lpPage);
    	r = Check(&hPixIF, lValue);
        PixDrvLink(&hPixIF,0);
        return r;
	}

    PixDrvLink(&hPixIF,0);

#ifdef INCLUDE_NOTES
    hOldCursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
	ReadAnnotations(szFile,*lpPage);
    SetCursor(hOldCursor);
#endif

	return PIX_SUCCESS;
}

int PixmToFile(HWND hWnd, PIXM_PAGE mPage)
{
	register int r;
    char szFile[PIX_FILE_MAXLEN];
    char szPipe[128];
	PixDrvHandle hDriver;
	INT32 lValue;
	HCURSOR hOldCursor;
	char *lpszSaveTypes = 0;
	PIXSAVESIMPLE ofn;
	struct TagListStruct tlOutputTags;

    /* Set all structure members to zero. */

    szFile[0] = 0;
    memset(&ofn, 0, sizeof(ofn));
    ofn.lStructSize = sizeof(ofn);
    ofn.hwndOwner = hWnd;
	ofn.hInstance = hInst;
    ofn.lpstrFile = szFile;
    ofn.nMaxFile = sizeof(szFile);
	ofn.lpPixfPack = &hPixfPack;
	ofn.lpOutputTags = &tlOutputTags;
	ofn.Flags = OFN_OVERWRITEPROMPT;
	ofn.dwPixFlags = PSS_ADDEXT;
	ofn.dwFileFormat = TAG_FILETYPE_TIFF ; /* default to TIFF */
	if (!PixSimpleSaveDlg(&ofn)) {
		return PIX_ERR_CANCEL;
	}

    if (r = Check(&hFromPixm, PixDrvInitialize(&hFromPixm, 0, (INT32)(void FAR *)mPage)))
        return r;

#if defined INCLUDE_NOTES && defined MERGE_NOTES
	if(NoteHndl[0])
		PixTagSetLong(&hFromPixm, TAG_ANNOTATION_HANDLE, 0, (long)NoteHndl[0]);
#endif

	szPipe[0] = 0;
	if (r = Check(&hFromPixm, AutoPipeIsis(&hFromPixm, &tlOutputTags, szPipe)))
		return r;

	/* save file in current orientation */
	PixmSettingsGet(pSettings, TAG_ORIENTATION, 0, &lValue);
	PixTagSetLong(&hFromPixm, TAG_ORIENTATION, 0, lValue);

	if (szPipe[0]) {
		if (r = Check(&hDriver, PixDrvLoadInitPipe(0, szPipe, &hDriver, 0))) {
			return r;
		}
    	PixDrvLink(&hFromPixm, &hDriver);
    }
    PixDrvAppend(&hFromPixm, &hPixfPack);

    PixTagSetAscii(&hFromPixm, TAG_OUTPUTNAME, szFile);
	PixTagSetLong(&hFromPixm, TAG_FILETYPE, 0, ofn.dwFileFormat);

    hOldCursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
    lValue = PixRunZone(&hFromPixm, buffer, sizeof buffer);
    SetCursor(hOldCursor);

    r = PIX_SUCCESS;
	if (lValue < 0) {
		PixFileUnlink(szFile,0);	/* make sure partial file deleted! */
    	r = Check(&hFromPixm, lValue);
	}

	/* take of PIXFPACK */
	PixDrvChopTail(&hFromPixm);
	if (szPipe[0]) {
		PixDrvLink(&hFromPixm, 0);
		PixDrvUnloadPipe(&hDriver);
	}

#if defined INCLUDE_NOTES && !defined MERGE_NOTES
    hOldCursor = SetCursor(LoadCursor(NULL, IDC_WAIT)); /* necessary? */
	SaveAnnotations(szFile);
    SetCursor(hOldCursor);
#endif
	return r;
}
