/****************************************************************************
    "SMPLSCAN.C"

    Copyright (c) 1993-95 Pixel Translations, Incorporated.
    All Rights Reserved.
    Confidential and Proprietary to Pixel Translations, Incorporated.

    Much of the scanning part of Smplview

*****************************************************************************/

/*>>keyflag<< * %n %f %l **/
/* SMPLSCAN.C 30-May-95,07:05:44  */

/*>>keyflag<< "&(#)%n v%v %f" */
static char szVersionId[] = "&(#)SMPLSCAN.C v11 30-May-95,07:05:44";

#ifndef WINDOWS
#define WINDOWS
#endif
#include <pixdflt.h>
#include <pixm.h>
#include <autopipe.h>
#include <pixsl.h>
#include <pixif.h>
#include "smplscan.h"
#include "smplfile.h"
#include "smplview.h"
#include <stdlib.h> 				/* for splitpath */
#include <commdlg.h>
#include <pixsvdlg.h>

extern PixDrvHandle hScan;			/* current scanner */
extern PixDrvHandle hFromPixm;		/* PIXM to ISIS driver */
extern PixDrvHandle hPixmIsis;		/* ISIS to PIXM driver */
extern PixDrvHandle hPixfPack;		/* ISIS PIXIF File Writer */

extern PIXM_PAGE mPage;				/* current page */
extern PIXM_SETTINGS pSettings;		/* current setttings */

extern char buffer[8192];			/* buffer used for numerous transfers */

static char lpcScanName[80];

/*
 *  if loaded, unload the main scanner driver
 *  returns PIX_SUCCESS always
 */

int ScanMaybeUnload(PixDrvHandlePtr lpDriver)
{
    if (lpDriver->lpCode) {
        PixDrvLink(lpDriver, 0);
        PixDrvUnload(lpDriver);
        lpDriver->lpCode = 0;
    }
    return PIX_SUCCESS;
}

/*
 *  if not already loaded, load & initialize main scanner driver
 */

int ScanMaybeLoad(HWND hWnd, PixDrvHandlePtr lpDriver)
{
    char lpcTemp[64];
    INT32 lStatus;
    HCURSOR hOldCursor;

    lStatus = PixDrvGetScanName(hWnd, lpcTemp); /* get scanner name */

    if ((*lpDriver).lpCode && !PixMemStrcmp(lpcTemp,lpcScanName)) /* already loaded; do nothing */
        return PIX_SUCCESS;

    hOldCursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
    ScanMaybeUnload(lpDriver);            /* unload previous driver */

    lStatus = PixDrvLoad(NULL,lpcTemp,lpDriver,0);
    if (lStatus < 0) {
        SetCursor(hOldCursor);
        (*lpDriver).lpCode = 0;
        Check(lpDriver, lStatus);
        return (int)lStatus;
    }

    PixMemStrcpy(lpcScanName,lpcTemp);

    lStatus = PixDrvInitialize(lpDriver,lpcTemp,0);
    SetCursor(hOldCursor);

    if (lStatus < 0) {
        Check(lpDriver, lStatus);
        ScanMaybeUnload(lpDriver);
        return HIWORD(lStatus);
    }

    return PIX_SUCCESS;
}

/*
 *	select a scanner
 */

int ScanSelect(HWND hWnd, PixDrvHandlePtr lpDriver)
{
	int r;

	ScanMaybeUnload(lpDriver);
	if (r = PixDrvSetScanName(hWnd))
		return r;
	return ScanMaybeLoad(hWnd, lpDriver);
}

/*
 *	scan page into PIXM
 *
 *	this version simply scans the page with PixRunZone for simplicity as
 *  an example.  That means you cannot cancel and the system is busy for
 *  the duration of the scan.  This is NOT necessary.  see the batch
 *  scanning routines for samples of how to yield CPU time and how to
 *  Cancel scanning too.
 */

int ScanToPixm(HWND hWnd, PixDrvHandlePtr lpScan, PIXM_PAGE FAR *lpNewPage)
{
	register int r;
	HCURSOR hOldCursor;
	INT32 lValue;

	if (r = ScanMaybeLoad(hWnd, lpScan))
		return r;

    PixTagSetLong(lpScan, TAG_SCANAHEAD, 0, 0L);

    if (r = Check(0,PixmPageNew(lpNewPage)))
    	return r;
    if (r = Check(&hPixmIsis, PixDrvInitialize(&hPixmIsis, 0,
			(INT32)(void FAR *)*lpNewPage))) {
        PixmPageFree(*lpNewPage);
        return r;
    }

    PixDrvLink(lpScan, &hPixmIsis);
    hOldCursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
    lValue = PixRunZone(lpScan, buffer, sizeof buffer);
    SetCursor(hOldCursor);

	if (lValue < 0) {
        PixmPageFree(*lpNewPage); /* new 921030 */
    	r = Check(lpScan, lValue);
        PixDrvLink(lpScan,0);
        return r;
	}
    PixDrvLink(lpScan, 0);

    return PIX_SUCCESS;
}

int
GetScannerTags(PixDrvHandlePtr lpScan, TAGLIST lpScannerTags)
{
    INT32  lBPS, lSPP, lPhoto, lPlanar;
	int iCurTag = 0;

	if (PixTagGetLong(lpScan, TAG_SAMPLESPERPIXEL, 0, &lSPP))
		lSPP = 1;
	lpScannerTags->tagValues[iCurTag].iTag = TAG_SAMPLESPERPIXEL;
	lpScannerTags->tagValues[iCurTag].lValue = 1;
	iCurTag++;

	if (PixTagGetLong(lpScan, TAG_BITSPERSAMPLE, 0, &lBPS))
		lBPS = 1;
	lpScannerTags->tagValues[iCurTag].iTag = TAG_BITSPERSAMPLE;
	lpScannerTags->tagValues[iCurTag].lValue = lBPS;
	iCurTag++;

	if (PixTagGetLong(lpScan, TAG_PHOTOMETRICINTERPRETATION, 0, &lPhoto)) {
		if (lSPP == 3)
			lPhoto = TAG_PHOTOMETRIC_RGB;
		else
			lPhoto = TAG_PHOTOMETRIC_WHITE0;
	}
	lpScannerTags->tagValues[iCurTag].iTag = TAG_PHOTOMETRICINTERPRETATION;
	lpScannerTags->tagValues[iCurTag].lValue = lPhoto;
	iCurTag++;

	if (PixTagGetLong(lpScan, TAG_PHOTOMETRICINTERPRETATION, 0, &lPhoto))
		lPlanar = TAG_PLANARCONFIG_LINE_PACK;
	lpScannerTags->tagValues[iCurTag].iTag = TAG_PLANARCONFIGURATION;
	lpScannerTags->tagValues[iCurTag].lValue = lPlanar;
	iCurTag++;

	lpScannerTags->tagValues[iCurTag].iTag = TAG_COMPRESSION;
	lpScannerTags->tagValues[iCurTag].lValue = TAG_COMPRESSION_NONE;
	iCurTag++;

	lpScannerTags->iCount = iCurTag;

return 0;
}

int GetExtension(INT32 dwFileFormat, char FAR *lpszExt)
{
	int i;
	INT16 iFormats;
	INT32 dwTestFormat;
	char szTmp[10];
	PixMemStrcpy(lpszExt, "xxx");
	if (PixChoiceGetCount(&hPixfPack, TAG_FILETYPE, &iFormats))
		return 0;
	for (i = 0; i < iFormats; i++) {
		PixChoiceGetLong(&hPixfPack, TAG_FILETYPE, i,
			&dwTestFormat);
		if (dwTestFormat == dwFileFormat) {
			PixChoiceGetAscii(&hPixfPack, TAG_AVAILFORMAT_EXTS, i, szTmp);
			PixMemStrcpy(lpszExt, &szTmp[2]);
			return 0;
		}
	}
	return 0;
}

int ScanBatch(HWND hWnd, PixDrvHandlePtr lpScan, PIXM_PAGE FAR *lpPage,
	PIXM_SETTINGS pSettingsi)
{
	register int r;
	char szPipe[128];
	char szFile[PIX_FILE_MAXLEN];
	char szTemp[PIX_FILE_MAXLEN];
	PixDrvHandle hDriver;
	PixDrvHandlePtr lpDriver = &hDriver;
	PIXSLJOB hJob;
	FARPROC lpfnSmplYield;
	char drive[_MAX_DRIVE];
	char dir[_MAX_DIR];
	char fname[_MAX_FNAME];
	char ext[_MAX_EXT];
	PIXDRVRUN hRun;
    PIXSAVESIMPLE ofn;
    PIXM_PAGE hNewPage;
	struct TagListStruct tlScannerTags;
	struct TagListStruct tlOutputTags;
	INT32 lValue;
	char szExt[10];

	if (r = ScanMaybeLoad(hWnd, lpScan))
		return r;

	GetScannerTags(lpScan, &tlScannerTags);

    /* get filename and filetype */

    szFile[0] = 0;
    memset(&ofn, 0, sizeof(ofn));
    ofn.lStructSize = sizeof(ofn);
    ofn.hwndOwner = hWnd;
	ofn.hInstance = hInst;
    ofn.lpstrFile = szFile;
    ofn.nMaxFile = sizeof(szFile);
	ofn.lpPixfPack = &hPixfPack;
	ofn.lpOutputTags = &tlOutputTags;
    ofn.Flags = OFN_OVERWRITEPROMPT;

	if (!PixSimpleSaveDlg(&ofn))
		return PIX_ERR_CANCEL;

	_splitpath(szFile, drive, dir, fname, ext);
	if (PixMemStrlen(fname) > 4) {
		MessageBox(hWnd, "Name must be at most four characters!",
			szAppName, MB_OK);
		return PIX_ERR_PARAM;
	}
	PixMemStrcpy(szFile, drive);
	PixMemStrcat(szFile, dir);
	PixMemStrcat(szFile, fname);

	/* Check for direct path, not modifying scanner settings */
	if (AutoPipeMake(&tlScannerTags, &tlOutputTags, szPipe) < 0) {
		/* Check for path, letting it modify the scanner settings */
		if ((r = AutoPipeIsis(lpScan, &tlOutputTags, szPipe)) < 0) {
			MessageBox(hWnd, "Couldn't create pipe to selected file "
				"format from this scanner.", szAppName, MB_OK);
			return r;
		} else {
			MessageBox(hWnd, "The settings on the scanner were changed to "
				"enable saving to the selected file format.\n", szAppName,
				MB_OK);
			r = 0;
		}
	}

	if (szPipe[0]) {
		if (r = Check(lpDriver, PixDrvLoadInitPipe(0,szPipe,lpDriver,0)))
			return r;
		PixDrvAppend(lpDriver, &hPixfPack);
	} else {
		lpDriver = &hPixfPack;
	}
	PixDrvLink(&hPixmIsis, lpDriver);

	if (r = Check(lpDriver, PixTagSetLong(lpDriver, TAG_FILETYPE, 0,
			ofn.dwFileFormat))) {
		PixDrvChopTail(&hPixmIsis);
		PixDrvLink(&hPixmIsis, 0);
		if (szPipe[0])
			PixDrvUnloadPipe(&hDriver);
		return r;
	}

    PixDrvLink(lpScan, &hPixmIsis);

    PixTagSetLong(lpScan, TAG_SCANAHEAD, 0, 1L);

	if (!(r = Check(0, PixslNewJob(&hJob)))) {
		PixslTagSet(hJob, PIXSL_TAG_PARENT, (INT32)hWnd);
		PixslTagSet(hJob, PIXSL_TAG_DRIVER, (INT32)(void FAR *)lpScan);
		PixslTagSet(hJob, PIXSL_TAG_AUTOSTART, 1L);
		PixslTagSet(hJob, PIXSL_TAG_APPNAME, (INT32)(char FAR *)szAppName);

		if (!(r = Check(0, PixslStartJob(hJob)))) {
			SmplDisableMenus(GetMenu(hWnd));
			lpfnSmplYield = MakeProcInstance((FARPROC)SmplYield, hInst);
			PixTagSetLong(lpScan, TAG_YIELDPROC, 0, (INT32)lpfnSmplYield);

			while (PixslPagePrepare(hJob) == PIXSL_SCAN) {
				PixslTagGet(hJob, PIXSL_TAG_PAGE, &lValue);
				GetExtension(ofn.dwFileFormat, szExt);
				if (lValue > 0)
					wsprintf(szTemp, "%s%03da.%s", (char FAR *)szFile, (int)-lValue,
						(char FAR *)szExt);
				else
					wsprintf(szTemp, "%s%03db.%s", (char FAR *)szFile, (int)lValue,
						(char FAR *)szExt);

				if (r = Check(lpScan, PixTagSetAscii(lpScan, TAG_OUTPUTNAME, szTemp)))
					break;

				if (r = Check(0, PixmPageNew(&hNewPage)))
					break;
				if (r = Check(0, PixDrvInitialize(&hPixmIsis, 0, (INT32)(void FAR *)hNewPage))) {
					PixmPageFree(hNewPage);
					break;
				}

				/* break up typical PixRunZone so yielding can occur even */
				/* on scanners which don't support TAG_YIELDPROC */

				if (!(lValue = PixDrvRunStart(lpScan, &hRun))) {
					while (1) {
						lValue = PixDrvRunStep(hRun, buffer, sizeof buffer, PIX_DRV_DATANOWAIT);
						if (lValue < 0)
							break;
						lValue = (*lpfnSmplYield)();
						if (lValue < 0)
							break;
					}
					if (lValue < 0 && HIWORD(lValue) != (UINT16)PIX_ERR_ENDZONE)
						PixDrvRunDone(hRun);
					else
						lValue = PixDrvRunDone(hRun);
				}

                if (r = Check(lpScan, lValue)) {
					PixslPageAbort(hJob);
					PixmPageFree(hNewPage);
					PixFileUnlink(szTemp, 0);	/* make sure partial page not saved */
					break;
                }

                if (*lpPage)
                    PixmPageFree(*lpPage);
	            *lpPage = hNewPage;
				SmplEnablePageMenus(GetMenu(hWnd));

				SetScrollRanges(hWnd, pSettings, *lpPage);
				InvalidateRect(hWnd, NULL, FALSE);
				UpdateWindow(hWnd);

				if (r = Check(0, PixslPageFinish(hJob)))
					break;
			}
		}

		PixslFreeJob(hJob);
	}

	PixTagSetLong(lpScan, TAG_YIELDPROC, 0, 0);
	SmplEnableMenus(GetMenu(hWnd));

	PixDrvSend(lpScan, PIX_DRV_SCANAHEADCLEAR, 0, 0); /* in case cancel */

	PixDrvChopTail(lpScan);
	PixDrvLink(lpScan,0);
	PixDrvLink(&hPixmIsis, 0);
	if (szPipe[0])
		PixDrvUnloadPipe(&hDriver);

	return r;
}

BOOL ValueIsLegal(PixDrvHandlePtr lpScan, int iTag, INT16 wCount,
	 INT32 lValue)
{
	int i;
	INT32 lTest;
	for (i=0; i<wCount; i++) {
		PixChoiceGetLong(lpScan, iTag, i, &lTest);
		if (lValue==lTest)
			return TRUE;
	}
	return FALSE;
}

void SetTag(PixDrvHandlePtr lpScan, int iTag, INT32 lValue, BOOL bCanBeZero)
{
	INT32 lLow, lHigh, lStep;
	PixChoiceGetLong(lpScan, iTag, PIXCHOICE_LOW , &lLow);
	PixChoiceGetLong(lpScan, iTag, PIXCHOICE_HIGH, &lHigh);
	PixChoiceGetLong(lpScan, iTag, PIXCHOICE_STEP, &lStep);

	/* too low? */
	if (lValue<lLow)
		lValue = lLow;
	/* too high? */
	if (lValue>lHigh)
		lValue = lHigh;
	/* not an multiple of lStep? */
	if (lValue-lLow % lStep)
		lValue = lLow + ( (lValue - lLow) / lStep ) * lStep ;
	/* Zero? */
	if (!lValue && !bCanBeZero)
		lValue = lStep;

	PixTagSetLong(lpScan, iTag, 0, lValue);
}

int ScanRotateBatch(HWND hWnd, PixDrvHandlePtr lpScan, PIXM_PAGE FAR *lpPage,
	PIXM_SETTINGS pSettings)
{
	register int r;
    PIXSAVESIMPLE ofn;
	char szPipe[128];
	char szExt[10];
    char szFile[PIX_FILE_MAXLEN];
	char szTemp[PIX_FILE_MAXLEN];
	PixDrvHandle hDriver;
	PixDrvHandlePtr lpDriver = &hDriver;
	PIXSLJOB hJob;
    INT32 lValue;
	INT32 lComp;
	char drive[_MAX_DRIVE];
	char dir[_MAX_DIR];
	char fname[_MAX_FNAME];
	char ext[_MAX_EXT];
	PIXDRVRUN hRun;
	PIXM_PAGE hNewPage;
	FARPROC lpfnSmplYield;
	BOOL bDuplex;
	int iPixslPage;
	INT16 wCount;
	struct TagListStruct tlOutputTags;
	INT32 lMaxWidth, lMaxLength, lXPosition, lYPosition, lWidth, lLength ;
	BOOL bSoftwareRotate = TRUE; /* this gets set to false if the scanner */
								 /* can rotate in firmware                */
	char szBuffer[128];

	if (r = ScanMaybeLoad(hWnd, lpScan))
		return r;

	/* The next section is used to determine whether or not Scan Rotate Batch
	   is a legal operation with the current scanner settings on the current
	   scanner */

	PixTagSaveValue(lpScan, 0);					/* save all tags */

	/* Check to see if an area is selected.  If so, warn the user that it */
	/* will be ignored                                                    */

	PixTagGetLong(lpScan, TAG_IMAGEWIDTH, 0, &lWidth);
	PixTagGetLong(lpScan, TAG_IMAGELENGTH, 0, &lLength);
	PixTagGetLong(lpScan, TAG_XPOSITION, 0, &lXPosition);
	PixTagGetLong(lpScan, TAG_YPOSITION, 0, &lYPosition);

	PixChoiceGetLong(lpScan, TAG_IMAGEWIDTH , PIXCHOICE_HIGH, &lMaxWidth );
	PixChoiceGetLong(lpScan, TAG_IMAGELENGTH, PIXCHOICE_HIGH, &lMaxLength);

	if (lXPosition || lYPosition || lWidth<lMaxWidth || lLength<lMaxLength) {
		r = MessageBox(hWnd,
				"A subarea is selected on the current page.  Scan "
				"Rotated Batch will ignore the selected subarea.  Continue?",
				szAppName, MB_OKCANCEL);
		if (r==IDCANCEL) {
			PixTagRestoreValue(lpScan, 0);
			return PIX_ERR_CANCEL;
		}
	}

	/* if TAG_SCANORIENTATION exists and TAG_SCANORIENTATION_LAND is an */
	/* available value, then the scanner can be told that the page is   */
	/* already in the scanner in landscape orientation.  At the time    */
	/* that this sample was written, only Fujitsu scanners through      */
	/* Kofax and Xionics ISIS drivers support this tag.					*/
	/* Also, if TAG_SCANORIENTATION exists but TAG_SCANORIENTATION_LAND */
	/* is not an allowable option, then the scanner cannot accept the   */
	/* current pagesize in landscape orientation, so error accordingly  */

	if (!PixChoiceGetCount(lpScan, TAG_SCANORIENTATION, &wCount)) {
		/* If TAG_SCANORIENTATION_LAND is legal, then use it */
		if (ValueIsLegal(lpScan, TAG_SCANORIENTATION,
				wCount, TAG_SCANORIENTATION_LAND)) {

			INT32 lValue;

			PixTagSetLong(lpScan, TAG_SCANORIENTATION,
						0, TAG_SCANORIENTATION_LAND);

			/* If the driver can report the orientation of the returned      */
			/* image and the returned image is portrait, then the scanner is */
			/* rotating the images and we don't have to do a thing.  At the  */
			/* same time, we set bSoftwareRotate to false so that we don't   */
			/* rotate the image again.  At the time of this sample's writing */
			/* only the Fujitsu 3099AG SCSI scanner supports this feature.   */

			if (!PixTagGetLong(lpScan, TAG_DATAORIENTATION, 0, &lValue) &&
					lValue == TAG_DATAORIENTATION_PORT) {

				/* scan & rotate is happening in the scanner.  Don't do it in */
				/* software													  */

				bSoftwareRotate = FALSE ;
				MessageBox(hWnd, "Scanner supports rotate-while-scan.  "
					"Application is not doing software rotation.",
					szAppName, MB_OK);
			}
		}
		else {
			MessageBox(hWnd, "Scanner cannot accept the current pagesize in "
				"landscape orientation.  Scan Rotated Batch is not available.",
				szAppName, MB_OK);

			PixTagRestoreValue(lpScan, 0);
			return PIX_ERR_CANCEL ;
		}
	}
	/* Scanner does not support TAG_SCANORIENTATION.  Attempt to calculate */
	/* a set of parameters that represents the current pagesize in         */
	/* landscape orientation                                               */
	else {
		int i;
		INT32 lWidthDifference, lStep;
		char szBestPagesize[128];

		/* if TAG_PAGESIZE is supported, then we calculate the width and   */
		/* length of the current pagesize, then we try different pagesizes */
		/* and pick a region in the best fitting pagesize that is the      */
		/* landscape equivalent of the originally selected pagesize.       */

		if (!PixChoiceGetCount(lpScan, TAG_PAGESIZE, &wCount)) {
			/* set X and Y position to zero so that the maximums for       */
			/* width and length come out correctly.                        */

			PixTagSetLong(lpScan, TAG_XPOSITION, 0, 0L);
			PixTagSetLong(lpScan, TAG_YPOSITION, 0, 0L);

			/* get maximum width and length for current pagesize           */

			PixChoiceGetLong(lpScan, TAG_IMAGEWIDTH ,
										PIXCHOICE_HIGH, &lWidth );
			PixChoiceGetLong(lpScan, TAG_IMAGELENGTH,
										PIXCHOICE_HIGH, &lLength);
			PixChoiceGetLong(lpScan, TAG_IMAGEWIDTH,
										PIXCHOICE_STEP, &lStep );
			/* it's ok to truncate what will be the desired width by the   */
			/* current width-step                                          */
			lLength = (lLength / lStep) * lStep ;

			/* lWidthDifference will be used to store the difference       */
			/* between the needed width and the best-fit pagesize's width. */
			/* Set it to unreasonably large so that it'll be overwritten   */
			/* while determining the best-fit pagesize                     */

			lWidthDifference = 1000000;

			for (i=0; i<wCount; i++) {
				PixChoiceGetAscii(lpScan, TAG_PAGESIZE, i, szBuffer);

				PixTagSetAscii(lpScan, TAG_PAGESIZE, szBuffer);

				PixChoiceGetLong(lpScan, TAG_IMAGEWIDTH ,
											PIXCHOICE_HIGH, &lMaxWidth );
				PixChoiceGetLong(lpScan, TAG_IMAGELENGTH,
											PIXCHOICE_HIGH, &lMaxLength );

				/* does the length of the old pagesize fit accross the     */
				/* width of this pagesize?                                 */
				/* does the width of the old pagesize fit accross the      */
				/* length of this pagesize?                                */

				if (lLength <= lMaxWidth && lWidth <= lMaxLength) {
					/* if this pagesize is the closest one yet, then store */
					/* it for use later                                    */
					if (lMaxWidth - lLength < lWidthDifference) {
						PixMemStrcpy(szBestPagesize, szBuffer);
						lWidthDifference = lMaxWidth - lLength;
					}
				}
			}
			/* if nothing overwrote lWidthDifference, then there's no      */
			/* pagesize that can fit the original pagesize in landscape    */
			/* orientation.                                                */
			if (lWidthDifference == 1000000) {
				MessageBox(hWnd, "Scanner cannot accept the current "
						"pagesize in landscape orientation.  Scan Rotated "
						"Batch is not available.", szAppName, MB_OK);
				PixTagRestoreValue(lpScan, 0);
				return PIX_ERR_CANCEL ;
			}

			/* Set TAG_PAGESIZE to the best-fit pagesize                   */
			PixTagSetAscii(lpScan, TAG_PAGESIZE, szBestPagesize);

			/* Set the desired width                                       */
			SetTag(lpScan, TAG_IMAGEWIDTH , lLength, FALSE);
			SetTag(lpScan, TAG_IMAGELENGTH, lWidth,  FALSE);

			PixTagGetLong(lpScan, TAG_FEEDER, 0, &lValue);
			if (lValue & TAG_FEEDER_CENTERFEED) {
				/* although it's not clear that I need to center because I */
				/* don't know if the page is coming from the flatbed or    */
				/* the feeder, center anyway on centerfeed scanners        */
				SetTag(lpScan, TAG_XPOSITION, lWidthDifference / 2, TRUE);
			}
		}
		else {
			/* Unless the scanner is square, it cannot support it's own    */
			/* maximum pagesize in landscape orientation, so error         */
			/* accordingly.                                                */
			MessageBox(hWnd, "Scanner cannot accept the current "
					"pagesize in landscape orientation.  Scan Rotated "
					"Batch is not available.", szAppName, MB_OK);
			PixTagRestoreValue(lpScan, 0);
			return PIX_ERR_CANCEL ;
		}
	}

    /* get filename and filetype */

    szFile[0] = 0;
    memset(&ofn, 0, sizeof(ofn));
    ofn.lStructSize = sizeof(ofn);
    ofn.hwndOwner = hWnd;
    ofn.lpstrFile = szFile;
    ofn.nMaxFile = sizeof(szFile);
    ofn.Flags = OFN_OVERWRITEPROMPT;
	ofn.lpPixfPack = &hPixfPack;
	ofn.lpOutputTags = &tlOutputTags;
	ofn.dwFileFormat = TAG_FILETYPE_TIFF;
	ofn.hInstance = hInst;

	if (!PixSimpleSaveDlg(&ofn))
		return PIX_ERR_CANCEL;

	_splitpath(szFile, drive, dir, fname, ext);
	if (PixMemStrlen(fname) > 4) {
		MessageBox(hWnd, "Name must be at most four characters!",
			szAppName, MB_OK);
		return PIX_ERR_PARAM;
	}
	PixMemStrcpy(szFile, drive);
	PixMemStrcat(szFile, dir);
	PixMemStrcat(szFile, fname);

    PixTagSetLong(lpScan, TAG_SCANAHEAD, 0, 1L);

	if (!(r = Check(0, PixslNewJob(&hJob)))) {
		PixslTagSet(hJob, PIXSL_TAG_PARENT, (INT32)hWnd);
		PixslTagSet(hJob, PIXSL_TAG_DRIVER, (INT32)(void FAR *)lpScan);
		PixslTagSet(hJob, PIXSL_TAG_AUTOSTART, 1L);
		PixslTagSet(hJob, PIXSL_TAG_APPNAME, (INT32)(char FAR *)szAppName);

		if (!(r = Check(0, PixslStartJob(hJob)))) {
			SmplDisableMenus(GetMenu(hWnd));
			lpfnSmplYield = MakeProcInstance((FARPROC)SmplYield, hInst);
			PixTagSetLong(lpScan, TAG_YIELDPROC, 0, (INT32)lpfnSmplYield);

			while (PixslPagePrepare(hJob) == PIXSL_SCAN) {

				if (r = Check(0, PixmPageNew(&hNewPage)))
					break;
				if (r = Check(0, PixDrvInitialize(&hPixmIsis, 0,
						(INT32)(void FAR *)hNewPage))) {
					PixmPageFree(hNewPage);
					break;
				}

				PixDrvLink(lpScan, &hPixmIsis);

				/* break up typical PixRunZone so yielding can occur even */
				/* on scanners which don't support TAG_YIELDPROC */

				if (!(lValue = PixDrvRunStart(lpScan, &hRun))) {
					while (1) {
						lValue = PixDrvRunStep(hRun, buffer,
							sizeof buffer, PIX_DRV_DATANOWAIT);
						if (lValue < 0)
							break;
						lValue = (*lpfnSmplYield)();
						if (lValue < 0)
							break;
					}
					if (lValue < 0 && HIWORD(lValue) != (UINT16)PIX_ERR_ENDZONE)
						PixDrvRunDone(hRun);
					else
						lValue = PixDrvRunDone(hRun);
				}

		        PixDrvLink(lpScan, 0);

		        if (r = Check(lpScan, lValue)) {
					PixmPageFree(hNewPage);
					break;
		        }

				/* Taken from mview */
				bDuplex = 0;
				if (!PixTagGetLong(lpScan, TAG_SCANTYPE, 0, &lValue))
					if (lValue == TAG_SCANTYPE_DUPLEX)
						bDuplex = 1;
				PixslTagGet(hJob, PIXSL_TAG_PAGE, &lValue);
				iPixslPage = (int)lValue;

				if (bSoftwareRotate && iPixslPage < 0 && bDuplex)
					PixmPageSet(hNewPage, TAG_ORIENTATION,TAG_ORIENTATION_LAND);
				else
				if (bSoftwareRotate)
					PixmPageSet(hNewPage, TAG_ORIENTATION, TAG_ORIENTATION_270);

		        if (*lpPage)
		            PixmPageFree(*lpPage);
		        if (*lpPage = hNewPage)
					SmplEnablePageMenus(GetMenu(hWndMain));

				SetScrollRanges(hWnd, pSettings, *lpPage);
				InvalidateRect(hWnd, NULL, FALSE);
				UpdateWindow(hWnd);

				if (r = Check(&hFromPixm, PixDrvInitialize(&hFromPixm, 0,
						(INT32)(void FAR *)*lpPage)))
					break;

				PixTagSetLong(&hFromPixm, TAG_ORIENTATION, 0,
					TAG_ORIENTATION_PORT);

				if (r = Check(&hFromPixm, PixAutoPipeIsis(&hFromPixm,
						&tlOutputTags, szPipe)))
		           	break;
				PixTagGetLong(&hFromPixm, TAG_COMPRESSION, 0, &lComp);

				lpDriver = &hDriver;
				if (szPipe[0]) {
					/* Load pipe */
					if (r = Check(0, PixDrvLoadInitPipe(0,szPipe,lpDriver,0)))
						break;
					PixDrvAppend(lpDriver, &hPixfPack);
				} else {
					lpDriver = &hPixfPack;
				}

				/* Tell PIXFPACK what file type to write */
				PixTagSetLong(lpDriver, TAG_FILETYPE, 0,
					ofn.dwFileFormat);

				PixslTagGet(hJob, PIXSL_TAG_PAGE, &lValue);
				GetExtension(ofn.dwFileFormat, szExt);
				if (lValue > 0)
					wsprintf(szTemp, "%s%03da.%s",
						(char FAR *)szFile, (int)-lValue,
						(char FAR *)szExt);
				else
					wsprintf(szTemp, "%s%03db.%s",
						(char FAR *)szFile, (int)lValue,
						(char FAR *)szExt);

				PixDrvLink(&hFromPixm, lpDriver);
				PixTagSetAscii(&hFromPixm, TAG_OUTPUTNAME, szTemp);

				/* no yield; fast enough? */
				lValue = PixRunZone(&hFromPixm, buffer, sizeof buffer);

				r = Check(&hFromPixm, lValue);

				/* remove PIXFPACK */
				PixDrvChopTail(&hFromPixm);
				PixDrvLink(&hFromPixm, 0);
				/* if there were other drivers, unload them */
				if (szPipe[0])
					PixDrvUnloadPipe(&hDriver);

				if (r)
					break;

				if (r = Check(0, PixslPageFinish(hJob)))
					break;

				r = PIX_SUCCESS;
			}
		}
		PixslFreeJob(hJob);
	}

	PixTagSetLong(lpScan, TAG_YIELDPROC, 0, 0);
	SmplEnableMenus(GetMenu(hWnd));

	PixDrvSend(lpScan, PIX_DRV_SCANAHEADCLEAR, 0, 0); /* in case cancel */

    PixTagRestoreValue(lpScan, 0);					/* restore all tags */

    return r;
}
