/*>>keyflag<< "%n"***********************************************************
	"SMPLVIEW.C"

	Copyright (c) 1993-96 Pixel Translations, Incorporated.
	All Rights Reserved.
	Confidential and Proprietary to Pixel Translations, Incorporated.

		This sample program was created in response to numerous requests for
		more sample source code for both PIXM and ISIS.  This program is very
		capable but is not intended to be a complete product.  Instead, it is
		intended to be a very solid example of the capabilities of ISIS and
		PIXM, and we hope it will help you during application development.

		See PVDEMO to get a better idea of some more advanced capabilities.

****************************************************************************
  TLIB Keywords: %v=version, %d=file date, %f=file date and time, %t=time
		 %w=who last, %l=lock status, %n=file name
*****************************************************************************/


/*>>keyflag<< * %n %f %l **/
/* SMPLVIEW.C 02-Jan-97,09:50:52  */

/*>>keyflag<< "&(#)%n v%v %f" */
static char szVersionId[] = "&(#)SMPLVIEW.C v40 02-Jan-97,09:50:52";

#ifndef OS2
#ifndef WINDOWS
#define WINDOWS     /* needed for PIXDFLT and PIXM */
#endif
#endif

#include <pixdflt.h>        /* ISIS header file (includes others) */
#include <pixloc.h>         /* Localization include file */
#include <pixm.h>           /* high-speed viewing (etc) module */

#include "smplview.h"       /* header for this sample */

#ifndef NO_SCAN
#include "smplscan.h"       /* some scanning routines for smplview */
#include <pixsl.h>          /* page sequencing for batch scans */
#endif

#include "smplprnt.h"       /* printing routines for smplview */
#include "smplfile.h"       /* file io routines for smplview */

#ifndef OS2
#include <commdlg.h>        /* windows common dialog functions */
#endif
#include <autopipe.h>       /* figure out correct ISIS pipe */

#include <pixpan.h>         /* pan window library */
#include "smplpan.h"        /* pan window functions */

PixDrvHandle hScan;         /* current scanner */
PixDrvHandle hFromPixm;     /* PIXM to ISIS driver */
PixDrvHandle hPixmIsis;     /* ISIS to PIXM driver */
PixDrvHandle hPixIF;        /* PIXIF file reader */
PixDrvHandle hPixfPack;     /* PIXIF file writer */

PIXM_PAGE mPage = 0;        /* current page */
UINT32    dwPagePalette = 0;/* added pj 931028 */
PIXM_SETTINGS pSettings = 0;/* current setttings */
extern PIXPAN hPixPan;      /* pan window handle */

BOOL bCancel;               /* user requested cancel of scan */
BOOL bBusy;                 /* busy scanning right now; can't exit etc. */

#ifdef INCLUDE_PIXM_TEST
BOOL bTestPattern=0;        /* Flag for WM_PAINT to draw test pattern
							   and not entire image */
BOOL bIsFulldrawn;          /* used with bTestPattern to know if the
							   test pattern is on the screen */
#endif

char buffer[8192];          /* buffer used for numerous transfers */
char szAppName[20];         /* class name for the window */
char szString[128];         /* variable to load resource strings */
HINSTANCE hInst;
HWND hWndMain;
HANDLE hAccel;
HWND hDlgWnd = 0;           /* for calibration dialog */

#if defined INCLUDE_NOTES && INCLUDE_NOTES==0
#undef INCLUDE_NOTES
#endif

#ifdef INCLUDE_NOTES

#include <pixif.h>
#include <pixnote.h>

#ifndef NOTECALLBACK
#define NOTECALLBACK 1
#endif

extern int ProcessNoteCmd(HWND hWnd, WPARAM wParam, LPARAM lParam);
extern void FixAnnoteMenu(HWND hWnd, PIXM_PAGE pPage);
extern PIXNOTE_HNDL NoteHndl[];
extern PIXNOTE_ENV NoteInstance;
HCURSOR ArrowCursor;
struct PixmPaintCallbackDesc rpcbDesc;
struct NoteArg_s NoteArgs;
#define FixMenus(win, page) FixAnnoteMenu(win, page)
#define DoNoteCmd(win, wP, lP) ProcessNoteCmd(win, wP, lP)
#else
#define FixMenus(win, page) 0
#define DoNoteCmd(win, wP, lP) 0
#endif

PIXM_PAINT_CALLBACKDESC prpcbRectCallBack= 0;

static void ParseCmdLine(LPSTR CmdStr);
static int SetOrientationCheck(HWND hWnd, PIXM_SETTINGS pSettings);
static int SetScaleCheck(HWND hWnd, PIXM_SETTINGS pSettings);

/*
	check an ISIS command.  if there was an error, display
	error string.  returns 16-bit ISIS error code.
 */

int Check(PixDrvHandlePtr lpDriver, INT32 lCode)
{
	static BOOL bRecurse = 0;

	if (lCode >= 0)
		return 0;

	if (!lpDriver && HIWORD(lCode)!=0xffff)
		lCode = MAKELONG(HIWORD(lCode),0xffff);

	if (HIWORD(lCode)==0xffff)      /* don't use driver unless custom code */
		lpDriver = (PixDrvHandlePtr)0;

	if (!bRecurse) {
		char outBuf[256];
		char drvName[256];
		char errStr[132];
		BOOL bDoModule = FALSE;

		bRecurse = 1;
		PixDrvGetError(NULL,lpDriver,lCode,errStr,sizeof errStr);

		/* if there is a driver(pipe), ask for the name of the */
		/* driver that returned the error */
		/* To do this, you ask the pipe for an error string, and */
		/* the error you pass it is MAKELONG(driverCode, PIX_ERR_VERSION). */
		/* driverCode is the low word of the original error */
		if (lpDriver) {
			INT32 r = PixDrvGetError(NULL, lpDriver,
				MAKELONG(LOWORD(lCode), PIX_ERR_VERSION), drvName,
				sizeof(drvName));
			bDoModule = (r >= 0);
		}

		if (bDoModule) {
			wsprintf(outBuf, "%s\n%s (%d)", (LPSTR)errStr,
				(LPSTR)drvName, (int)(short)((HIWORD(lCode) == -1)
					? LOWORD(lCode)
					: HIWORD(lCode)));
		} else {
			wsprintf(outBuf, "%s (%d)", (LPSTR)errStr, (int)(short)((HIWORD(lCode) == (WORD)-1)
				? LOWORD(lCode)
				: HIWORD(lCode)));
		}

		MessageBox(0, outBuf, "Smplview", MB_OK);
		bRecurse = 0;
	}

	return (HIWORD(lCode) == 0xffff)
		? LOWORD(lCode)
		: HIWORD(lCode);
}

/*
	set checkmarks on Orientation menu
 */

#ifdef OS2
#else
static int SetOrientationCheck(HWND hWnd, PIXM_SETTINGS pSettings)
{
	INT32 lValue;

	PixmSettingsGet(pSettings, TAG_ORIENTATION, 0, &lValue);

	CheckMenuItem(GetMenu(hWnd), IDM_O_MIRROR, MF_BYCOMMAND |
		(lValue >= 5 ? MF_CHECKED : MF_UNCHECKED));

	lValue = (lValue-1) & 3;

	CheckMenuItem(GetMenu(hWnd), IDM_O_0DEGREESPORTRAIT, MF_BYCOMMAND |
		(lValue == 0 ? MF_CHECKED : MF_UNCHECKED));
	CheckMenuItem(GetMenu(hWnd), IDM_O_90DEGREESLANDSCAPE, MF_BYCOMMAND |
		(lValue == 1 ? MF_CHECKED : MF_UNCHECKED));
	CheckMenuItem(GetMenu(hWnd), IDM_O_180DEGREES, MF_BYCOMMAND |
		(lValue == 2 ? MF_CHECKED : MF_UNCHECKED));
	CheckMenuItem(GetMenu(hWnd), IDM_O_270DEGREES, MF_BYCOMMAND |
		(lValue == 3 ? MF_CHECKED : MF_UNCHECKED));

	return 0;
}
#endif

/*
	set checkmarks on Scaling menu
 */

#ifdef OS2
#else
static int SetScaleCheck(HWND hWnd, PIXM_SETTINGS pSettings)
{
	INT32 lValue;

	PixmSettingsGet(pSettings, TAG_SCALE_X, 0, &lValue);

	CheckMenuItem(GetMenu(hWnd), IDM_Z_ALLPIXELS11, MF_BYCOMMAND |
		(lValue == TAG_SCALE_NONE ? MF_CHECKED : MF_UNCHECKED));
	CheckMenuItem(GetMenu(hWnd), IDM_Z_FITTOWIDTH, MF_BYCOMMAND |
		(lValue == TAG_SCALE_FITX ? MF_CHECKED : MF_UNCHECKED));
	CheckMenuItem(GetMenu(hWnd), IDM_Z_FITTOWINDOW, MF_BYCOMMAND |
		(lValue == TAG_SCALE_FITBOTH ? MF_CHECKED : MF_UNCHECKED));

	return 0;
}
#endif

#define INT32STRING "%ld"

int CheckAndSet(HWND hWnd, int iTag, char FAR *lpcBuf, char FAR *lpcDescribe)
{
	INT32 lStep;
	INT32 lValue1, lValue2;
	char buffer[50];

	PixChoiceGetLong(&hScan,iTag,PIXCHOICE_STEP,&lStep);
	lValue1 = (PixScanDecimalLong(lpcBuf,0)/lStep)*lStep; /* round down */
	/* might be better to round width, length up, but ... this is */
	/* just a test program! */

	PixChoiceGetLong(&hScan,iTag,PIXCHOICE_LOW,&lValue2);
	if (lValue1 < lValue2) {
		wsprintf(buffer,"Value should be at least " INT32STRING, lValue2);
		MessageBox(hWnd,buffer,lpcDescribe,MB_OK);
		return PIX_ERR_PARAM;
	}

	PixChoiceGetLong(&hScan,iTag,PIXCHOICE_HIGH,&lValue2);
	if (lValue1 > lValue2) {
		wsprintf(buffer,"Value should be at most " INT32STRING, lValue2);
		MessageBox(hWnd,buffer,lpcDescribe,MB_OK);
		return PIX_ERR_PARAM;
	}

	return (PixTagSetLong(&hScan,iTag,0,lValue1)) ? 1 : 0;
}

#ifndef NO_SCAN
BOOL PUBLIC_FUNC ScanAreaProc(HWND hDlg, UINT message,
	WPARAM wParam, LPARAM lParam);

BOOL PUBLIC_FUNC ScanAreaProc(HWND hDlg, UINT message,
		WPARAM wParam, LPARAM lParam)
{
	char buf[50];
	INT32 lValue;

	switch (message) {
		case WM_INITDIALOG:        /* message: initialize dialog box */
			PixTagGetLong(&hScan,TAG_XPOSITION,0,&lValue);
			wsprintf(buf,INT32STRING,lValue);
			SetWindowText( GetDlgItem(hDlg, IDC_XPOS),   buf);
			PixTagGetLong(&hScan,TAG_YPOSITION,0,&lValue);
			wsprintf(buf,INT32STRING,lValue);
			SetWindowText( GetDlgItem(hDlg, IDC_YPOS),   buf);
			PixTagGetLong(&hScan,TAG_IMAGEWIDTH,0,&lValue);
			wsprintf(buf,INT32STRING,lValue);
			SetWindowText( GetDlgItem(hDlg, IDC_WIDTH),  buf);
			PixTagGetLong(&hScan,TAG_IMAGELENGTH,0,&lValue);
			wsprintf(buf,INT32STRING,lValue);
			SetWindowText( GetDlgItem(hDlg, IDC_HEIGHT), buf);
			return (TRUE);

		case WM_COMMAND:              /* message: received a command */
			switch (GET_WM_COMMAND_ID(wParam, lParam)) {
				case IDOK:
					GetDlgItemText(hDlg,IDC_XPOS,  buf,sizeof buf);
					if (CheckAndSet(hDlg,TAG_XPOSITION,buf,"X Position"))
					   break;
					GetDlgItemText(hDlg,IDC_YPOS,  buf,sizeof buf);
					if (CheckAndSet(hDlg,TAG_YPOSITION,buf,"Y Position"))
					   break;
					GetDlgItemText(hDlg,IDC_WIDTH, buf,sizeof buf);
					if (CheckAndSet(hDlg,TAG_IMAGEWIDTH,buf,"Image Width"))
					   break;
					GetDlgItemText(hDlg,IDC_HEIGHT,buf,sizeof buf);
					if (CheckAndSet(hDlg,TAG_IMAGELENGTH,buf,"Image Length"))
					   break;
					EndDialog(hDlg, TRUE);   /* Exits the dialog box      */
					return TRUE;

				case IDCANCEL:
					EndDialog(hDlg, FALSE);  /* Exits the dialog box      */
					return TRUE;

#ifdef HALFSCAN
				case 200: /* top half */
					PixTagSetLong(&hScan,TAG_XPOSITION,0,0L);
					PixChoiceGetLong(&hScan,TAG_IMAGEWIDTH,PIXCHOICE_HIGH,&lValue);
					PixTagSetLong(&hScan,TAG_IMAGEWIDTH,0,lValue);

					PixTagSetLong(&hScan,TAG_YPOSITION,0,0L);
					PixChoiceGetLong(&hScan,TAG_IMAGELENGTH,PIXCHOICE_HIGH,&lValue);
					PixTagSetLong(&hScan,TAG_IMAGELENGTH,0,lValue/2);
					ScanAreaProc(hDlg, WM_INITDIALOG,0,0); /* update text */
					break;
				case 201: /* bottom half */
					PixTagSetLong(&hScan,TAG_XPOSITION,0,0L);
					PixChoiceGetLong(&hScan,TAG_IMAGEWIDTH,PIXCHOICE_HIGH,&lValue);
					PixTagSetLong(&hScan,TAG_IMAGEWIDTH,0,lValue);

					PixTagSetLong(&hScan,TAG_YPOSITION,0,0);
					PixChoiceGetLong(&hScan,TAG_IMAGELENGTH,PIXCHOICE_HIGH,&lValue);
					PixTagSetLong(&hScan,TAG_YPOSITION,0,lValue/2);
					PixChoiceGetLong(&hScan,TAG_IMAGELENGTH,PIXCHOICE_HIGH,&lValue);
					PixTagSetLong(&hScan,TAG_IMAGELENGTH,0,lValue);
					ScanAreaProc(hDlg, WM_INITDIALOG,0,0); /* update text */
					break;
#endif

				default:
					break;

			}
			break;
	}
	return (FALSE);               /* Didn't process a message    */
}

#endif

/*
	the following functions allow SMPLVIEW to multitask within WINDOWS
	this must be done carefully to avoid reentrancy problems
*/

#ifdef OS2
#else
int SmplDisableMenus(HMENU hMenu) /* during scan disable */
{
	bCancel = FALSE;
	bBusy = TRUE;

	EnableMenuItem(hMenu, IDM_F_CANCELSCAN, MF_BYCOMMAND|MF_ENABLED);

	EnableMenuItem(hMenu, IDM_F_OPENFILE, MF_BYCOMMAND|MF_GRAYED);
	EnableMenuItem(hMenu, IDM_F_SELECTSCANNER, MF_BYCOMMAND|MF_GRAYED);
	EnableMenuItem(hMenu, IDM_F_SETSCANPARAM, MF_BYCOMMAND|MF_GRAYED);
	EnableMenuItem(hMenu, IDM_F_SCANPAGE, MF_BYCOMMAND|MF_GRAYED);
	EnableMenuItem(hMenu, IDM_F_SCANBATCH, MF_BYCOMMAND|MF_GRAYED);
	EnableMenuItem(hMenu, IDM_F_SCANROTBATCH, MF_BYCOMMAND|MF_GRAYED);
	EnableMenuItem(hMenu, IDM_F_SCANAREA, MF_BYCOMMAND|MF_GRAYED);

	EnableMenuItem(hMenu, IDM_F_CALIBRATE, MF_BYCOMMAND|MF_GRAYED);

	return PIX_SUCCESS;
}
#endif

#ifdef OS2
#else
int SmplEnableMenus(HMENU hMenu) /* not scanning, enable */
{
	bBusy = FALSE;

	EnableMenuItem(hMenu, IDM_F_CANCELSCAN, MF_BYCOMMAND|MF_GRAYED);

	EnableMenuItem(hMenu, IDM_F_OPENFILE, MF_BYCOMMAND|MF_ENABLED);
	EnableMenuItem(hMenu, IDM_F_SELECTSCANNER, MF_BYCOMMAND|MF_ENABLED);
	EnableMenuItem(hMenu, IDM_F_SETSCANPARAM, MF_BYCOMMAND|MF_ENABLED);
	EnableMenuItem(hMenu, IDM_F_SCANPAGE, MF_BYCOMMAND|MF_ENABLED);
	EnableMenuItem(hMenu, IDM_F_SCANBATCH, MF_BYCOMMAND|MF_ENABLED);
	EnableMenuItem(hMenu, IDM_F_SCANROTBATCH, MF_BYCOMMAND|MF_ENABLED);
	EnableMenuItem(hMenu, IDM_F_SCANAREA, MF_BYCOMMAND|MF_ENABLED);

	EnableMenuItem(hMenu, IDM_F_CALIBRATE, MF_BYCOMMAND|MF_ENABLED);

	return PIX_SUCCESS;
}
#endif

/*
	don't allow some operations if no pages loaded
*/

#ifdef OS2
#else
int SmplEnablePageMenus(HMENU hMenu) /* have a page loaded */
{
	EnableMenuItem(hMenu,
		IDM_F_CLOSEFILE, MF_BYCOMMAND|MF_ENABLED);
	EnableMenuItem(hMenu,
		IDM_F_PRINT, MF_BYCOMMAND|MF_ENABLED);
#ifdef INCLUDE_PRINT_TEST
	EnableMenuItem(hMenu,
		IDM_F_PRINT_TEST, MF_BYCOMMAND|MF_ENABLED);
#endif
	EnableMenuItem(hMenu,
		IDM_F_PRINT11, MF_BYCOMMAND|MF_ENABLED);
	EnableMenuItem(hMenu,
		IDM_F_SAVEFILE, MF_BYCOMMAND|MF_ENABLED);

	return PIX_SUCCESS;
}
#endif

#ifdef OS2
#else
int SmplDisablePageMenus(HMENU hMenu) /* no page loaded; can't do following */
{
	EnableMenuItem(hMenu,
		IDM_F_CLOSEFILE, MF_BYCOMMAND|MF_GRAYED);
	EnableMenuItem(hMenu,
		IDM_F_PRINT, MF_BYCOMMAND|MF_GRAYED);
#ifdef INCLUDE_PRINT_TEST
	EnableMenuItem(hMenu,
		IDM_F_PRINT_TEST, MF_BYCOMMAND|MF_GRAYED);
#endif
	EnableMenuItem(hMenu,
		IDM_F_PRINT11, MF_BYCOMMAND|MF_GRAYED);
	EnableMenuItem(hMenu,
		IDM_F_SAVEFILE, MF_BYCOMMAND|MF_GRAYED);

	return PIX_SUCCESS;
}
#endif

/*
	simple function sets scrollbar ranges.  ideally this function
	would turn on and off scroll bars as needed too.

	if you are looking for something to speed up, look at this.
	turning on and off scroll bars (or manipulating them in general)
	is incredibly slow under windows.
 */
#ifdef OS2
#else
int SetScrollRanges(HWND hWnd, PIXM_SETTINGS pSettings, PIXM_PAGE pPage)
{
	INT32 lValue;
	UINT16 wPageWidth, wPageLen;
	RECT Rect;
	PIXM_SETTINGS pTemp;
	int i, r;

	if (!pPage)
		return FALSE;

	/* get basic values */

	GetClientRect(hWnd, &Rect);
	PixmPageGet(pPage, TAG_IMAGEWIDTH, &lValue);
	wPageWidth = (UINT16)lValue;
	PixmPageGet(pPage, TAG_IMAGELENGTH, &lValue);
	wPageLen = (UINT16)lValue;
	PixmSettingsGet(pSettings, TAG_ORIENTATION, 0, &lValue);
	if ((lValue-1) & 1) { /* swap */
		wPageWidth ^= wPageLen;
		wPageLen ^= wPageWidth;
		wPageWidth ^= wPageLen;
	}

	/* a temporary settings and PixmRectPaint are used to get the actual
	   scale factor used in case ImageAccel loaded */

	PixmSettingsNew(&pTemp, 1);
	PixmSettingsCopy(pTemp, 0, pSettings, 0, 1);
	r = PixmRectPaint(hWnd, 0, pPage, pTemp, 0,
		0, 0, 0, 0,
		PIXM_PAINT_FAST|PIXM_PAINT_ACCEL|PIXM_PAINT_NODRAW,
		0, prpcbRectCallBack);

	if (r < 0) { /* can't continue if scale didn't get computed*/
		PixmSettingsFree(pTemp);
		return r;
	}

	/* scale raw original image */

	PixmSettingsGet(pTemp, TAG_SCALE_X, 0, &lValue);
	wPageWidth = (UINT16)((long)wPageWidth *
		(long)PIXM_RAT_NUM(lValue) / (long)PIXM_RAT_DEN(lValue));
	PixmSettingsGet(pTemp, TAG_SCALE_Y, 0, &lValue);
	wPageLen = (UINT16)((long)wPageLen *
		(long)PIXM_RAT_NUM(lValue) / (long)PIXM_RAT_DEN(lValue));

	/* ready to figure out range */

	i = (int)wPageWidth - (int)Rect.right;
	PixmSettingsGet(pSettings, TAG_XPOSITION, 0, &lValue);
	if (lValue > i) {
#ifndef SLIDING_WINDOWS
		lValue = (i>0)?i:0;
#endif
		PixmSettingsSet(pSettings, TAG_XPOSITION, 0, 0, lValue);
	}
	SetScrollRange(hWnd, SB_HORZ, 0, (i>0)?i:0, TRUE);

	i = (int)wPageLen - (int)Rect.bottom;
	PixmSettingsGet(pSettings, TAG_YPOSITION, 0, &lValue);
	if (lValue > i) {
#ifndef SLIDING_WINDOWS
		lValue = (i>0)?i:0;
#endif
		PixmSettingsSet(pSettings, TAG_YPOSITION, 0, 0, lValue);
	}
	SetScrollRange(hWnd, SB_VERT, 0, (i>0)?i:0, TRUE);

	PixmSettingsFree(pTemp);

	return TRUE;
}
#endif

int PUBLIC_FUNC WinMain(HINSTANCE hInstance, HANDLE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow);

#ifdef OS2
int main() {
	/* TODO Where are the parameters */
	return WinMain(0, 0, NULL, 0);
	}
#endif

int PUBLIC_FUNC WinMain(HINSTANCE hInstance, HANDLE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow)
{
	 /***********************************************************************/
	 /* HINSTANCE hInstance;    handle for this instance                    */
	 /* HANDLE hPrevInstance;   handle for possible previous instances      */
	 /* LPSTR  lpszCmdLine;     long pointer to exec command line           */
	 /* int    nCmdShow;        Show code for main window display           */
	 /***********************************************************************/

	MSG		msg;	/* MSG structure to store your messages		*/
	int		nRc;	/* return value from Register Classes		*/

	ParseCmdLine(lpszCmdLine);

	PixdfltInit();

	strcpy(szAppName, "SMPLVIEW");
	hInst = hInstance;
	if(!hPrevInstance) {
		/* register window classes if first instance of application         */
		if ((nRc = nCwRegisterClasses()) == -1) {
			/* registering one of the windows failed                         */
			LoadString(hInst, IDS_ERR_REGISTER_CLASS, szString, sizeof(szString));
			MessageBox(NULL, szString, NULL, MB_ICONEXCLAMATION);
			PixdfltDone();
			return nRc;
		}
	}

	/* create application's Main window                                    */
	hWndMain = CreateWindow(
				szAppName,               /* Window class name           */
				"Simple Viewer",         /* Window's title              */
				WS_CAPTION      |        /* Title and Min/Max           */
				WS_SYSMENU      |        /* Add system menu box         */
				WS_MINIMIZEBOX  |        /* Add minimize box            */
				WS_MAXIMIZEBOX  |        /* Add maximize box            */
				WS_THICKFRAME   |        /* thick sizeable frame        */
				WS_VSCROLL      |        /* add vertical scroll bar     */
				WS_HSCROLL      |        /* add horizontal scroll bar   */
				WS_CLIPCHILDREN |         /* don't draw in child windows areas */
				WS_OVERLAPPED,
				CW_USEDEFAULT, 0,        /* Use default X, Y            */
				CW_USEDEFAULT, 0,        /* Use default X, Y            */
				NULL,                    /* Parent window's handle      */
				NULL,                    /* Default to Class Menu       */
				hInst,                   /* Instance of window          */
				NULL);                   /* Create struct for WM_CREATE */


	if(hWndMain == NULL) {
		LoadString(hInst, IDS_ERR_CREATE_WINDOW, szString, sizeof(szString));
		MessageBox(NULL, szString, NULL, MB_ICONEXCLAMATION);
		PixdfltDone();
		return IDS_ERR_CREATE_WINDOW;
	}
#ifdef INCLUDE_NOTES
	ArrowCursor= LoadCursor(0,IDC_ARROW);
	PixNoteEnvNew(&NoteInstance);
	rpcbDesc.lpfnCallback= PixNotePixmPaintCB;
	rpcbDesc.lOptions= PIXM_CALLBACK_OFFSCREENIFCONVENIENT;
	rpcbDesc.lParam= (INT32)(int FAR*)NoteHndl;
#if NOTECALLBACK
	prpcbRectCallBack= &rpcbDesc;
#endif
#endif
	PrintInitialize();                  /* initialize printing variables */
	PixPanInitialize();                 /* initialize pan window stuff */
	CreatePanWindow(hWndMain);

	hPixmIsis.lpCode = 0;               /* load ISIS to PIXM driver */
	if (Check(0,PixDrvLoad(0,"PIXMISIS", &hPixmIsis, 0))) {
		PixdfltDone();
		return 0;
	}

										/* load PIXIF file input driver */
	hPixIF.lpCode = 0;
	if (Check(0, PixDrvLoadInitPipe(0, "F_PIXIF", &hPixIF, 0))) {
		PixDrvUnload(&hPixmIsis);
		PixdfltDone();
		return 0;
	}

										/* load PIXFPACK file output driver */
	hPixfPack.lpCode = 0;
	if (Check(0, PixDrvLoadInitPipe(0, "PIXFPACK", &hPixfPack, 0))) {
		PixDrvUnload(&hPixIF);
		PixDrvUnload(&hPixmIsis);
		PixdfltDone();
		return 0;
	}

										/* load PIXM to ISIS driver */
	if (Check(0,PixDrvLoad(0,"FROMPIXM", &hFromPixm, 0))) {
		PixDrvUnload(&hPixIF);
		PixDrvUnload(&hPixfPack);
		PixDrvUnload(&hPixmIsis);
		PixdfltDone();
		return 0;
	}

	if (Check(0,AutoPipeInit())) {      /* initialize AUTOPIPE */
		PixDrvUnload(&hPixIF);
		PixDrvUnload(&hPixfPack);
		PixDrvUnload(&hFromPixm);
		PixDrvUnload(&hPixmIsis);
		PixdfltDone();
		return 0;
	}

	PixmInitialize();                   /* initialize PIXM */
	PixmSettingsNew(&pSettings, 1);
	PixmSettingsSet(pSettings, TAG_SCALE_X, 0, 0, TAG_SCALE_FITBOTH);

	ShowWindow(hWndMain, nCmdShow);            /* display main window      */
	UpdateWindow(hWndMain);                    /* HELP ?? */

	hAccel = LoadAccelerators(hInst, szAppName);

	SetOrientationCheck(hWndMain, pSettings); /* setup menus */
	SetScaleCheck(hWndMain, pSettings);
	SmplEnableMenus(GetMenu(hWndMain));
	SmplDisablePageMenus(GetMenu(hWndMain));
	CheckMenuItem(GetMenu(hWndMain), IDM_L_FIRST, MF_BYCOMMAND | MF_CHECKED);

	while(GetMessage(&msg, NULL, 0, 0)) {        /* Until WM_QUIT message    */
		if (!(hDlgWnd && IsDialogMessage(hDlgWnd, &msg))) {
			if(TranslateAccelerator(hWndMain, hAccel, &msg))
				continue;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	/* Do clean up before exiting from the application                     */
	CwUnRegisterClasses();

#ifdef INCLUDE_NOTES
	PixNoteEnvFree(NoteInstance);
#endif

	if (hPixmIsis.lpCode)
		PixDrvUnload(&hPixmIsis);

	if (hFromPixm.lpCode)
		PixDrvUnload(&hFromPixm);

	if (hPixIF.lpCode)
		PixDrvUnload(&hPixIF);

	if (hPixfPack.lpCode)
		PixDrvUnload(&hPixfPack);

	ScanMaybeUnload(&hScan);
	AutoPipeDone();

	/* remove PIXM_PAGE and PIXM_SETTINGS from memory */

	if (mPage) {
		PixmPageFree(mPage);
		PixmPaletteRelease(dwPagePalette);
		dwPagePalette = 0;
	}
	if (pSettings)
		PixmSettingsFree(pSettings);

	PixmDone();
	PrintDone();
	PixdfltDone();

	return msg.wParam;
} /*  End of WinMain                                                    */


static void ParseCmdLine(LPSTR CmdStr) {
	for(;*CmdStr;CmdStr++) {
		if(!lstrcmpi(CmdStr,"/debug")) {
		}
	}
}

int PUBLIC_FUNC SmplYield(void)
{
	MSG msg;
	while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
		if(TranslateAccelerator(hWndMain, hAccel, &msg))
			continue;
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return bCancel ? PIX_ERR_CANCEL : PIX_SUCCESS;
}


/************************************************************************/
/*                                                                      */
/* Main Window Procedure                                                */
/*                                                                      */
/* This procedure provides service routines for the Windows events      */
/* (messages) that Windows sends to the window, as well as the user     */
/* initiated events (messages) that are generated when the user selects */
/* the action bar and pulldown menu controls or the corresponding       */
/* keyboard accelerators.                                               */
/*                                                                      */
/************************************************************************/

LONG PUBLIC_FUNC WndProc(HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	HMENU       hMenu=0;            /* handle for the menu                 */
	HBITMAP     hBitmap=0;          /* handle for bitmaps                  */
	HDC         hDC;                /* handle for the display device       */
	PAINTSTRUCT ps;                 /* holds PAINT information             */
	int         nRc=0;              /* return code                         */
	INT32       lValue, lValueTmp;
	RECT		rectClient;
	int         i,j,r;
	PIXM_PAGE   hNewPage;
	UINT32		lLang = 0;
	int			lIndex = 0;

#ifdef INCLUDE_NOTES
	if(PixNoteIsMsg(NoteInstance,hWnd,Message,wParam,lParam) & PIXISMSG_CONSUMED)
		return(0);
	if(Message==WM_MOUSEMOVE)
		SetCursor(ArrowCursor);
#endif
	switch (Message) {

		case WM_QUERYENDSESSION:
			return bBusy ? 0 : 1;

		case WM_COMMAND:

			switch (GET_WM_COMMAND_ID(wParam,lParam)) {
#ifdef INCLUDE_NOTES
				/*CHP: Annotation commands */
				case IDM_N_LOAD:
				case IDM_N_EDIT:
				case IDM_N_SAVE:
				case IDM_N_CLOSE:
				case IDM_N_NEW:
				case IDM_N_REDRAW:
				case IDM_N_PIXIF:
					DoNoteCmd(hWnd,wParam,lParam);
					FixMenus(hWnd, mPage);
					break;
#endif
				/* file io functions */

				case IDM_F_CLOSEFILE:
					if (mPage) {
						if (DoNoteCmd(hWnd, IDM_N_CLOSE, 0L))
							break;
						PixmPageFree(mPage);
						PixmPaletteRelease(dwPagePalette);
						dwPagePalette = 0;
						mPage = 0;
						SmplDisablePageMenus(GetMenu(hWndMain));
						NewImagePanWindow();
						FixMenus(hWnd, mPage);
						InvalidateRect(hWnd, NULL, FALSE);
					}
					break;

				case IDM_F_OPENFILE:
					if (mPage) {
						if (DoNoteCmd(hWnd, IDM_N_CLOSE, 0L))
							break;
					}
					if (!FileToPixm(hWnd, &hNewPage)) {
						HDC hDC;
						if (mPage) {
							PixmPageFree(mPage);
							PixmPaletteRelease(dwPagePalette);
							dwPagePalette = 0;
						}
						if (mPage = hNewPage) {
							hDC = GetDC(hWnd);
							PixmPaletteFromPages(hWnd, &mPage, 1, &dwPagePalette);
							ReleaseDC(hWnd, hDC);
							SmplEnablePageMenus(GetMenu(hWndMain));
						}

						SetScrollRanges(hWnd, pSettings, mPage);
						NewImagePanWindow();
						FixMenus(hWnd, mPage);
						InvalidateRect(hWnd, NULL, FALSE);
					}
					break;

				case IDM_F_SAVEFILE:
					PixmToFile(hWnd, mPage);
					break;

				/* scanning functions */

				case IDM_F_SCANAREA: {
					FARPROC lpProcSetup;
					if (ScanMaybeLoad(hWnd, &hScan))
						break;

					lpProcSetup = MakeProcInstance(ScanAreaProc, hInst);

					DialogBox(hInst,        /* current instance     */
						"ZONING",           /* resource to use      */
						hWnd,               /* parent handle        */
						lpProcSetup);       /* About() instance address */

					FreeProcInstance(lpProcSetup);
					}
					break;


				case IDM_F_SELECTSCANNER:
					ScanSelect(hWnd, &hScan);
					break;

				case IDM_F_SETSCANPARAM:
					if (!ScanMaybeLoad(hWnd, &hScan)) {
						PixDrvSetDialog(&hScan, hWnd, hInst, 0);
					}
					break;

				case IDM_F_SCANPAGE: /* scan into PIXM */
					if (DoNoteCmd(hWnd, IDM_N_CLOSE, 0L))
						break;
					if (!ScanToPixm(hWnd, &hScan, &hNewPage)) {
						if (mPage) {
							PixmPageFree(mPage);
							PixmPaletteRelease(dwPagePalette);
							dwPagePalette = 0;
						}
						if (mPage = hNewPage) {
							HDC hDC = GetDC(hWnd);
							PixmPaletteFromPages(hWnd, &mPage, 1, &dwPagePalette);
							ReleaseDC(hWnd, hDC);
							SmplEnablePageMenus(GetMenu(hWndMain));
						}

						SetScrollRanges(hWnd, pSettings, mPage);
						NewImagePanWindow();
						InvalidateRect(hWnd, NULL, FALSE);
					}
					break;

				case IDM_F_CANCELSCAN: /* only implemented on batch scans */
					bCancel = TRUE;
					break;

				case IDM_F_SCANBATCH:
					if (DoNoteCmd(hWnd, IDM_N_CLOSE, 0L))
						break;
					ScanBatch(hWnd, &hScan, &mPage, pSettings);
					break;

				case IDM_F_SCANROTBATCH:
					if (DoNoteCmd(hWnd, IDM_N_CLOSE, 0L))
						break;
					ScanRotateBatch(hWnd, &hScan, &mPage, pSettings);
					break;

				case IDM_F_CALIBRATE:
					if (ScanMaybeLoad(hWnd, &hScan))
						break;
					if (PixTagGetLong(&hScan, TAG_CALIBRATION_DLG, 0, &lValue) || !lValue) {
						MessageBox(hWnd, "This scanner does not support calibration.", "Error", MB_OK);
						break;
					}
					hDlgWnd = (HWND)HIWORD(PixDrvCalibrateDlg(&hScan, hWnd, IDM_F_SCANPAGE, IDM_F_STOPCALIBRATE));
					break;

				case IDM_F_STOPCALIBRATE:
					hDlgWnd = 0;
					break;

				case IDM_F_EXIT:
					PostMessage(hWnd, WM_CLOSE, 0, 0);
					break;

				/* printing functions */

				case IDM_F_PRINT:
					if (mPage)
						PrintPage(mPage, pSettings, hWnd, szAppName, TAG_SCALE_FITBOTH);
					break;

#ifdef INCLUDE_PRINT_TEST
				case IDM_F_PRINT_TEST:
					if (mPage)
						PrintTest(mPage, pSettings, hWnd, szAppName, TAG_SCALE_FITBOTH);
					break;
#endif
				case IDM_F_PRINT11:
					if (mPage)
						PrintPage(mPage, pSettings, hWnd, szAppName, TAG_SCALE_NONE);
					break;

				case IDM_F_SETUPPRINT:
					PrintSetup(hWnd);
					break;

				/* view setting functions */

				case IDM_Z_ALLPIXELS11:
					/* Place User Code to respond to the                   */
					/* Menu Item Named "All Pixels (1:1)" here.            */
					PixmSettingsSet(pSettings, TAG_SCALE_X, 0, 0, TAG_SCALE_NONE);
					InvalidateRect(hWnd, NULL, FALSE);
					SetScaleCheck(hWnd, pSettings);
					SetScrollRanges(hWnd, pSettings, mPage);
					break;

				case IDM_Z_FITTOWINDOW:
					/* Place User Code to respond to the                   */
					/* Menu Item Named "Fit to Window" here.               */
					PixmSettingsSet(pSettings, TAG_SCALE_X, 0, 0, TAG_SCALE_FITBOTH);
					InvalidateRect(hWnd, NULL, FALSE);
					SetScaleCheck(hWnd, pSettings);
					SetScrollRanges(hWnd, pSettings, mPage);
					break;

				case IDM_Z_FITTOWIDTH:
					/* Place User Code to respond to the                   */
					/* Menu Item Named "Fit to Width" here.                */
					PixmSettingsSet(pSettings, TAG_SCALE_X, 0, 0, TAG_SCALE_FITX);
					InvalidateRect(hWnd, NULL, FALSE);
					SetScaleCheck(hWnd, pSettings);
					SetScrollRanges(hWnd, pSettings, mPage);
					break;

				case IDM_Z_ZOOMIN:
					if (!mPage)
						break;

					/* Place User Code to respond to the                   */
					/* Menu Item Named "Zoom In" here.                     */
					r = PixmRectPaint(hWnd, 0, mPage, pSettings, 0, 0, 0, 0, 0,
						PIXM_PAINT_ACCEL|PIXM_PAINT_FAST|PIXM_PAINT_NODRAW,
						0, 0);

					if (r < 0) { /*can't continue if scale didn't get computed*/
						MessageBeep(0);  /* just beep on error;  */
						break;
					}

					PixmSettingsGet(pSettings, TAG_SCALE_X, 0, &lValue);
					/* normalize scale factor to n/100 */
					lValue = PIXM_RAT_NUM(lValue) * 100L / PIXM_RAT_DEN(lValue);
					lValue += 10;
					if (lValue > 400)
					   lValue = 400;
					lValue = PIXM_RAT(LOWORD(lValue), 100);

					PixmSettingsSet(pSettings, TAG_SCALE_X, 0, 0, lValue);
					InvalidateRect(hWnd, NULL, FALSE);
					SetScaleCheck(hWnd, pSettings);
					SetScrollRanges(hWnd, pSettings, mPage);
					break;

				case IDM_Z_ZOOMOUT:
					if (!mPage)
						break;

					/* Place User Code to respond to the                   */
					/* Menu Item Named "Zoom Out" here.                    */
					r = PixmRectPaint(hWnd, 0, mPage, pSettings, 0, 0, 0, 0, 0,
						PIXM_PAINT_ACCEL|PIXM_PAINT_FAST|PIXM_PAINT_NODRAW,
						0, 0);

					if (r < 0) { /*can't continue if scale didn't get computed*/
						MessageBeep(0);  /* just beep on error;  */
						break;
					}

					PixmSettingsGet(pSettings, TAG_SCALE_X, 0, &lValue);
					/* normalize scale factor to n/100 */
					lValue = PIXM_RAT_NUM(lValue) * 100L / PIXM_RAT_DEN(lValue);
					lValue -= 10;
					if (lValue < 10)
					   lValue = 10;
					lValue = PIXM_RAT(LOWORD(lValue), 100);

					PixmSettingsSet(pSettings, TAG_SCALE_X, 0, 0, lValue);
					InvalidateRect(hWnd, NULL, FALSE);
					SetScaleCheck(hWnd, pSettings);
					SetScrollRanges(hWnd, pSettings, mPage);
					break;

				case IDM_O_0DEGREESPORTRAIT:
					/* Place User Code to respond to the                   */
					/* Menu Item Named "0-degrees (Portrait)" here.        */
					PixmSettingsGet(pSettings, TAG_ORIENTATION, 0, &lValue);
					lValue = TAG_ORIENTATION_PORT + ((lValue >= 5) ? 4 : 0);
					PixmSettingsSet(pSettings, TAG_ORIENTATION, 0, 0, lValue);
					InvalidateRect(hWnd, NULL, FALSE);
					SetOrientationCheck(hWnd, pSettings);
					SetScrollRanges(hWnd, pSettings, mPage);
					break;

				case IDM_O_90DEGREESLANDSCAPE:
					/* Place User Code to respond to the                   */
					/* Menu Item Named "90-degress (Landscape)" here.      */
					PixmSettingsGet(pSettings, TAG_ORIENTATION, 0, &lValue);
					lValue = TAG_ORIENTATION_LAND + ((lValue >= 5) ? 4 : 0);
					PixmSettingsSet(pSettings, TAG_ORIENTATION, 0, 0, lValue);
					InvalidateRect(hWnd, NULL, FALSE);
					SetOrientationCheck(hWnd, pSettings);
					SetScrollRanges(hWnd, pSettings, mPage);
					break;

				case IDM_O_180DEGREES:
					/* Place User Code to respond to the                   */
					/* Menu Item Named "180-degrees" here.                 */
					PixmSettingsGet(pSettings, TAG_ORIENTATION, 0, &lValue);
					lValue = TAG_ORIENTATION_180 + ((lValue >= 5) ? 4 : 0);
					PixmSettingsSet(pSettings, TAG_ORIENTATION, 0, 0, lValue);
					InvalidateRect(hWnd, NULL, FALSE);
					SetOrientationCheck(hWnd, pSettings);
					SetScrollRanges(hWnd, pSettings, mPage);
					break;

				case IDM_O_270DEGREES:
					/* Place User Code to respond to the                   */
					/* Menu Item Named "270-degrees" here.                 */
					PixmSettingsGet(pSettings, TAG_ORIENTATION, 0, &lValue);
					lValue = TAG_ORIENTATION_270 + ((lValue >= 5) ? 4 : 0);
					PixmSettingsSet(pSettings, TAG_ORIENTATION, 0, 0, lValue);
					InvalidateRect(hWnd, NULL, FALSE);
					SetOrientationCheck(hWnd, pSettings);
					SetScrollRanges(hWnd, pSettings, mPage);
					break;

				case IDM_O_MIRROR:
					/* Place User Code to respond to the                   */
					/* Menu Item Named "270-degrees" here.                 */
					PixmSettingsGet(pSettings, TAG_ORIENTATION, 0, &lValue);
					lValue = ((lValue-1) ^ 4)+1;
					PixmSettingsSet(pSettings, TAG_ORIENTATION, 0, 0, lValue);
					InvalidateRect(hWnd, NULL, FALSE);
					SetOrientationCheck(hWnd, pSettings);
					break;


				case IDM_O_SCALETOGRAY:
					/* Menu Item Named "Scale to Gray" here       */
					PixmSettingsGet(pSettings, TAG_SCALING, 0, &lValue);
					switch(lValue) {
						case TAG_SCALING_GRAY4:
							lValue = TAG_SCALING_BINARY;
							break;
						case TAG_SCALING_BINARY:
							lValue = TAG_SCALING_GRAY4;
							break;
					}
					if (lValue == TAG_SCALING_GRAY4 ||
						lValue == TAG_SCALING_BINARY) {
						PixmSettingsSet(pSettings, TAG_SCALING, 0, 0, lValue);
						InvalidateRect(hWnd, NULL, FALSE);
						CheckMenuItem(GetMenu(hWnd), IDM_O_SCALETOGRAY, MF_BYCOMMAND |
							(lValue == TAG_SCALING_GRAY4 ? MF_CHECKED : MF_UNCHECKED));
					}

					/* change menus for horiz/vert preserve or bright/contrast */

					if (lValue == TAG_SCALING_GRAY4) {
						ModifyMenu(GetMenu(hWnd), IDM_I_MOREBLACKHORIZONTAL,
							MF_BYCOMMAND|MF_STRING, IDM_I_MOREBLACKHORIZONTAL,
							"Increase &Black (darken)\tB");
						ModifyMenu(GetMenu(hWnd), IDM_I_MOREWHITEHORIZONTAL,
							MF_BYCOMMAND|MF_STRING, IDM_I_MOREWHITEHORIZONTAL,
							"Increase &White (lighten)\tW");
						ModifyMenu(GetMenu(hWnd), IDM_I_MOREBLACKVERTICAL,
							MF_BYCOMMAND|MF_STRING, IDM_I_MOREBLACKVERTICAL,
							"&Increase Contrast\tCtrl B");
						ModifyMenu(GetMenu(hWnd), IDM_I_MOREWHITEVERTICAL,
							MF_BYCOMMAND|MF_STRING, IDM_I_MOREWHITEVERTICAL,
							"&Decrease Contrast\tCtrl W");
					} else if (lValue == TAG_SCALING_BINARY) {
						ModifyMenu(GetMenu(hWnd), IDM_I_MOREBLACKHORIZONTAL,
							MF_BYCOMMAND|MF_STRING, IDM_I_MOREBLACKHORIZONTAL,
							"More &Black (horizontal)\tB");
						ModifyMenu(GetMenu(hWnd), IDM_I_MOREWHITEHORIZONTAL,
							MF_BYCOMMAND|MF_STRING, IDM_I_MOREWHITEHORIZONTAL,
							"More &White (horizontal)\tW");
						ModifyMenu(GetMenu(hWnd), IDM_I_MOREBLACKVERTICAL,
							MF_BYCOMMAND|MF_STRING, IDM_I_MOREBLACKVERTICAL,
							"More Black (vertical)\tCtrl B");
						ModifyMenu(GetMenu(hWnd), IDM_I_MOREWHITEVERTICAL,
							MF_BYCOMMAND|MF_STRING, IDM_I_MOREWHITEVERTICAL,
							"More White (vertical)\tCtrl W");
					}
					break;

				case IDM_O_INVERT:
					/* Place User Code to respond to the                   */
					/* Menu Item Named "Invert" here.                      */
					PixmSettingsGet(pSettings, TAG_INVERT, 0, &lValue);
					lValue = 1-lValue;
					PixmSettingsSet(pSettings, TAG_INVERT, 0, 0, lValue);
					InvalidateRect(hWnd, NULL, FALSE);
					CheckMenuItem(GetMenu(hWnd), IDM_O_INVERT, MF_BYCOMMAND |
						(lValue ? MF_CHECKED : MF_UNCHECKED));
					break;

				case IDM_O_OVERSCANCOLOR:
					/* Place User Code to respond to the                   */
					/* Menu Item Named "Overscan Color" here.              */
					PixmSettingsGet(pSettings, TAG_OVERSCAN, 0, &lValue);
					PixmSettingsSet(pSettings, TAG_OVERSCAN, 0, 0, 1-lValue);
					InvalidateRect(hWnd, NULL, FALSE);
					CheckMenuItem(GetMenu(hWnd), IDM_O_OVERSCANCOLOR, MF_BYCOMMAND |
						(lValue == 0 ? MF_CHECKED : MF_UNCHECKED));
					break;

				case IDM_O_BITSWAP:             /* Bitswap menu choice */
					PixmSettingsGet(pSettings, TAG_FILLORDER, 0, &lValue);
					PixmSettingsSet(pSettings, TAG_FILLORDER, 0, 0, 3-lValue);
					InvalidateRect(hWnd, NULL, FALSE);
					CheckMenuItem(GetMenu(hWnd), IDM_O_BITSWAP, MF_BYCOMMAND |
						(lValue == 1 ? MF_CHECKED : MF_UNCHECKED));
					break;

				case IDM_O_PANWINDOW:               /* PanWindow menu choice */
					/* turn pan window off if on, on if off */
					if (hPixPan && (PixPanIsVisible(hPixPan))) {
						PixPanClose(hPixPan);
						CheckMenuItem(GetMenu(hWnd), IDM_O_PANWINDOW,
							MF_BYCOMMAND | MF_UNCHECKED);
					}
					else {
						ShowPanWindow();
						CheckMenuItem(GetMenu(hWnd), IDM_O_PANWINDOW,
							MF_BYCOMMAND | MF_CHECKED);
					}
					break;

				case IDM_I_MOREBLACKHORIZONTAL:
					if (!mPage)
						break;
					/* Place User Code to respond to the                   */
					/* Menu Item Named "More black (horizontal)" here.     */

					/* handles x threshold if binary scaling and
					   brightness if scale to gray */
					if ((PixmPageGet(mPage, TAG_SAMPLESPERPIXEL, &lValue)
						!= PIX_SUCCESS) ||
						(PixmPageGet(mPage, TAG_BITSPERSAMPLE, &lValueTmp)
						!= PIX_SUCCESS) ||
						(lValue == 1 && lValueTmp < 8)) {
						lValueTmp = 1;
					} else
						lValueTmp = 0;
					if (lValueTmp == 1) {
						PixmSettingsGet(pSettings, TAG_SCALING, 0, &lValue);
						if (lValue == TAG_SCALING_BINARY) {
							PixmSettingsGet(pSettings, TAG_THRESH_X, 0, &lValue);
							lValue += 16;
							if (lValue > TAG_THRESH_X_MAX)
								lValue = TAG_THRESH_X_MAX;
							PixmSettingsSet(pSettings, TAG_THRESH_X, 0, 0, lValue);
						} else {
							lValueTmp = 0;
						}
					}
					if (!lValueTmp) {
						PixmSettingsGet(pSettings, TAG_BRIGHTNESS, 0, &lValue);
						lValue += 16;
						if (lValue >= PIXM_BRIGHTNESS_MAX)
							lValue = PIXM_BRIGHTNESS_MAX;
						PixmSettingsSet(pSettings, TAG_BRIGHTNESS, 0, 0, lValue);
					}
					GetClientRect(hWnd, &rectClient);
					if (PixmRectPaint(hWnd, 0,            /* DC, Window */
						mPage, pSettings, 0,              /* image data & settings */
						0, &rectClient,                   /* repaint area & posn */
						0, 0,                             /* no buffer passed */
						PIXM_PAINT_FAST|PIXM_PAINT_ACCEL| /* use Image Accel if avail */
						  PIXM_PAINT_NOMODIFY|dwPagePalette|PIXM_PAINT_ADJUSTFORSETTINGS,
						0, prpcbRectCallBack))                            /* options */
						MessageBeep(0);                  /* just beep on error; avoid
														   infinite loop of repaints */
					break;

				case IDM_I_MOREWHITEHORIZONTAL:
					if (!mPage)
						break;

					/* Place User Code to respond to the                   */
					/* Menu Item Named "More white (horizontal)" here.     */

					/* handles x threshold if binary scaling and
					   brightness if scale to gray */

					if ((PixmPageGet(mPage, TAG_SAMPLESPERPIXEL, &lValue)
						!= PIX_SUCCESS) ||
						(PixmPageGet(mPage, TAG_BITSPERSAMPLE, &lValueTmp)
						!= PIX_SUCCESS) ||
						(lValue == 1 && lValueTmp < 8)) {
						lValueTmp = 1;
					} else
						lValueTmp = 0;
					if (lValueTmp == 1) {
						PixmSettingsGet(pSettings, TAG_SCALING, 0, &lValue);
						if (lValue == TAG_SCALING_BINARY) {
							PixmSettingsGet(pSettings, TAG_THRESH_X, 0, &lValue);
							lValue -= 16;
							if (lValue < TAG_THRESH_X_MIN)
								lValue = TAG_THRESH_X_MIN;
							PixmSettingsSet(pSettings, TAG_THRESH_X, 0, 0, lValue);
						} else {
							lValueTmp = 0;
						}
					}
					if (!lValueTmp) {
						PixmSettingsGet(pSettings, TAG_BRIGHTNESS, 0, &lValue);
						lValue -= 16;
						if (lValue < PIXM_BRIGHTNESS_MIN)
							lValue = PIXM_BRIGHTNESS_MIN;
						PixmSettingsSet(pSettings, TAG_BRIGHTNESS, 0, 0, lValue);
					}
					GetClientRect(hWnd, &rectClient);
					if (PixmRectPaint(hWnd, 0,            /* DC, Window */
						mPage, pSettings, 0,              /* image data & settings */
						0, &rectClient,                   /* repaint area & posn */
						0, 0,                             /* no buffer passed */
						PIXM_PAINT_FAST|PIXM_PAINT_ACCEL| /* use Image Accel if avail */
						  PIXM_PAINT_NOMODIFY|dwPagePalette|PIXM_PAINT_ADJUSTFORSETTINGS,
						0, prpcbRectCallBack))                            /* options */
						MessageBeep(0);                  /* just beep on error; avoid
														   infinite loop of repaints */
					break;

				case IDM_I_MOREBLACKVERTICAL:
					if (!mPage)
						break;
					/* Place User Code to respond to the                   */
					/* Menu Item Named "More black (vertical)" here.       */
					if ((PixmPageGet(mPage, TAG_SAMPLESPERPIXEL, &lValue)
						!= PIX_SUCCESS) ||
						(PixmPageGet(mPage, TAG_BITSPERSAMPLE, &lValueTmp)
						!= PIX_SUCCESS) ||
						(lValue == 1 && lValueTmp < 8)) {
						lValueTmp = 1;
					} else
						lValueTmp = 0;
					if (lValueTmp == 1) {
						PixmSettingsGet(pSettings, TAG_SCALING, 0, &lValue);
						if (lValue == TAG_SCALING_BINARY) {
							PixmSettingsGet(pSettings, TAG_THRESH_Y, 0, &lValue);
							lValue += 1;
							if (lValue > TAG_THRESH_Y_MAX)
								lValue = TAG_THRESH_Y_MAX;
							PixmSettingsSet(pSettings, TAG_THRESH_Y, 0, 0, lValue);
						} else {
							lValueTmp = 0;
						}
					}
					if (!lValueTmp) {
						PixmSettingsGet(pSettings, TAG_CONTRAST, 0, &lValue);
						lValue += 16;
						if (lValue >= PIXM_CONTRAST_MAX)
							lValue = PIXM_CONTRAST_MAX;
						PixmSettingsSet(pSettings, TAG_CONTRAST, 0, 0, lValue);
					}
					GetClientRect(hWnd, &rectClient);
					if (PixmRectPaint(hWnd, 0,            /* DC, Window */
						mPage, pSettings, 0,              /* image data & settings */
						0, &rectClient,                   /* repaint area & posn */
						0, 0,                             /* no buffer passed */
						PIXM_PAINT_FAST|PIXM_PAINT_ACCEL| /* use Image Accel if avail */
						  PIXM_PAINT_NOMODIFY|dwPagePalette|PIXM_PAINT_ADJUSTFORSETTINGS,
						0, prpcbRectCallBack))                            /* options */
						MessageBeep(0);                  /* just beep on error; avoid
														   infinite loop of repaints */
					break;

				case IDM_I_MOREWHITEVERTICAL:
					if (!mPage)
						break;
					/* Place User Code to respond to the                   */
					/* Menu Item Named "More white (vertical)" here.       */
					if ((PixmPageGet(mPage, TAG_SAMPLESPERPIXEL, &lValue)
						!= PIX_SUCCESS) ||
						(PixmPageGet(mPage, TAG_BITSPERSAMPLE, &lValueTmp)
						!= PIX_SUCCESS) ||
						(lValue == 1 && lValueTmp < 8)) {
						lValueTmp = 1;
					} else
						lValueTmp = 0;
					if (lValueTmp == 1) {
						PixmSettingsGet(pSettings, TAG_SCALING, 0, &lValue);
						if (lValue == TAG_SCALING_BINARY) {
						   PixmSettingsGet(pSettings, TAG_THRESH_Y, 0, &lValue);
						   lValue -= 1;
						   if (lValue < TAG_THRESH_Y_MIN)
							   lValue = TAG_THRESH_Y_MIN;
						   PixmSettingsSet(pSettings, TAG_THRESH_Y, 0, 0, lValue);
						} else {
							lValueTmp = 0;
						}
					}
					if (!lValueTmp) {
					   PixmSettingsGet(pSettings, TAG_CONTRAST, 0, &lValue);
					   lValue -= 16;
					   if (lValue < PIXM_CONTRAST_MIN)
						   lValue = PIXM_CONTRAST_MIN;
					   PixmSettingsSet(pSettings, TAG_CONTRAST, 0, 0, lValue);
					}
					GetClientRect(hWnd, &rectClient);
					if (PixmRectPaint(hWnd, 0,            /* DC, Window */
						mPage, pSettings, 0,              /* image data & settings */
						0, &rectClient,                   /* repaint area & posn */
						0, 0,                             /* no buffer passed */
						PIXM_PAINT_FAST|PIXM_PAINT_ACCEL| /* use Image Accel if avail */
						  PIXM_PAINT_NOMODIFY|dwPagePalette|PIXM_PAINT_ADJUSTFORSETTINGS,
						0, prpcbRectCallBack))                            /* options */
						MessageBeep(0);                  /* just beep on error; avoid
														   infinite loop of repaints */
					break;

				case IDM_I_DEFAULTS:
					PixmSettingsSet(pSettings, TAG_THRESH_X, 0, 0, PIXM_THRESH_X_DEF);
					PixmSettingsSet(pSettings, TAG_THRESH_Y, 0, 0, PIXM_THRESH_Y_DEF);
					PixmSettingsSet(pSettings, TAG_BRIGHTNESS, 0, 0, PIXM_BRIGHTNESS_DEF);
					PixmSettingsSet(pSettings, TAG_CONTRAST, 0, 0, PIXM_CONTRAST_DEF);
					InvalidateRect(hWnd, NULL, FALSE);
					break;

				/* These all fall through to the last one */
				case IDM_L_ENGLISH: if (!lLang) lLang = 0x409;
				case IDM_L_FRENCH: if (!lLang) lLang = 0x40C;
				case IDM_L_SPANISH: if (!lLang) lLang = 0x40A;
				case IDM_L_ITALIAN: if (!lLang) lLang = 0x410;
				case IDM_L_GERMAN: if (!lLang) lLang = 0x407;
				case IDM_L_SWEDISH: if (!lLang) lLang = 0x41D;
					for (i = IDM_L_FIRST; i <= IDM_L_LAST; i++) {
						if (i == (int)GET_WM_COMMAND_ID(wParam,lParam)) {
							CheckMenuItem(GetMenu(hWnd), i,
								MF_BYCOMMAND | MF_CHECKED);
						} else {
							CheckMenuItem(GetMenu(hWnd), i,
								MF_BYCOMMAND | MF_UNCHECKED);
						}
					}
					PixLocalizeSetLanguage(lLang);
					break;


#ifdef INCLUDE_PIXM_TEST
				case IDM_T_PATTERN: /* Compile the .RC file with debugging
									   on, and this option will be in the
									   menu */
					bTestPattern=!bTestPattern;
					if (bTestPattern) {
						SetTimer(hWnd, 1, 2000, NULL);
						bIsFulldrawn=1;
						PixmTestReset(hWnd, mPage, pSettings);
						InvalidateRect(hWnd, NULL, FALSE);
						UpdateWindow(hWnd);
					}
					else {
						KillTimer(hWnd, 1);
/*
						PixmTestReset(hWnd, mPage, pSettings);
						InvalidateRect(hWnd, NULL, FALSE);
						UpdateWindow(hWnd);*/

					}
					break;
#endif

				case IDM_A_ABOUTSMPLVIEW:
					/* Place User Code to respond to the                   */
					/* Menu Item Named "About Smplview..." here.           */
					{
						FARPROC lpfnABOUTMsgProc;

						lpfnABOUTMsgProc = MakeProcInstance((FARPROC)ABOUTMsgProc, hInst);
						nRc = DialogBox(hInst, "SmplAboutDlg", hWnd, lpfnABOUTMsgProc);
						FreeProcInstance(lpfnABOUTMsgProc);
					}
					break;

				default:
					return DefWindowProc(hWnd, Message, wParam, lParam);
			}
			break;        /* End of WM_COMMAND                             */

#ifdef INCLUDE_PIXM_TEST
		case WM_LBUTTONDOWN:
			if (bTestPattern) {
				SendMessage(hWnd, WM_COMMAND, IDM_T_PATTERN, 0L);
				return 0;
			}
			else
				return DefWindowProc(hWnd, Message, wParam, lParam);

		case WM_TIMER:
			if (!mPage) {
				KillTimer(hWnd,1);
				bTestPattern = 0;
			}
			if (bTestPattern) {
				if (bIsFulldrawn) { /* redraw with the pattern */
					KillTimer(hWnd,1);
					bIsFulldrawn=0;
					InvalidateRect(hWnd, NULL, FALSE);
					UpdateWindow(hWnd);
					SetTimer(hWnd, 1, 2000, NULL);
				}
				else {        /* change settings, then redraw fullscreen */
					if (!PixmTestNext(hWnd, mPage, pSettings))
						SendMessage(hWnd, WM_COMMAND, IDM_T_PATTERN, 0L);
					else {
						KillTimer(hWnd,1);
						bIsFulldrawn=1;
						InvalidateRect(hWnd, NULL, FALSE);
						UpdateWindow(hWnd);
						SetTimer(hWnd, 1, 2000, NULL);
					}
				}
			}
			break;
#endif

		case WM_CREATE:
			break;       /*  End of WM_CREATE                              */

		case WM_MOVE:     /*  code for moving the window                    */
			break;

		case WM_SIZE:     /*  code for sizing client area                   */
			SetScrollRanges(hWnd, pSettings, mPage);
			break;       /* End of WM_SIZE                                 */

		case WM_CHAR:
			switch (wParam) {
				case 'C':
				case 'c':
					SendMessage(hWnd, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDM_F_CANCELSCAN, 0, 0));
					break;

				case '1':
					SendMessage(hWnd, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDM_O_0DEGREESPORTRAIT, 0,0));
					break;
				case '2':
					SendMessage(hWnd, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDM_O_90DEGREESLANDSCAPE, 0,0));
					break;
				case '3':
					SendMessage(hWnd, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDM_O_180DEGREES, 0,0));
					break;
				case '4':
					SendMessage(hWnd, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDM_O_270DEGREES, 0,0));
					break;
				case 'm':
				case 'M':
					SendMessage(hWnd, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDM_O_MIRROR, 0,0));
					break;

				case 'g':
				case 'G':
					SendMessage(hWnd, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDM_O_SCALETOGRAY, 0,0));
					break;
				case 'i':
				case 'I':
					SendMessage(hWnd, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDM_O_INVERT, 0,0));
					break;
				case 'o':
				case 'O':
					SendMessage(hWnd, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDM_O_OVERSCANCOLOR, 0,0));
					break;
				case 'r':
				case 'R':
					SendMessage(hWnd, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDM_O_BITSWAP, 0,0));
					break;
				case 'p':
				case 'P':
					SendMessage(hWnd, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDM_O_PANWINDOW, 0,0));
					break;

				case '+':
					SendMessage(hWnd, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDM_Z_ZOOMIN, 0,0));
					break;
				case '-':
					SendMessage(hWnd, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDM_Z_ZOOMOUT, 0,0));
					break;

				case 'b':
				case 'B':
					SendMessage(hWnd, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDM_I_MOREBLACKHORIZONTAL, 0,0));
					break;
				case 'w':
				case 'W':
					SendMessage(hWnd, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDM_I_MOREWHITEHORIZONTAL, 0,0));
					break;
				case 'B'-'A'+1:
					SendMessage(hWnd, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDM_I_MOREBLACKVERTICAL, 0,0));
					break;
				case 'W'-'A'+1:
					SendMessage(hWnd, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDM_I_MOREWHITEVERTICAL, 0,0));
					break;
				case 'n':
				case 'N':
					SendMessage(hWnd, WM_COMMAND,
						GET_WM_COMMAND_MPS(IDM_I_DEFAULTS, 0,0));
					break;
#ifdef SLIDING_WINDOWS
				case 'z':
				case 'Z':
					PixmSettingsGet(pSettings, TAG_YPOSITION, 0, &lValue);
					PixmSettingsSet(pSettings, TAG_YPOSITION, 0, 0, lValue - 20);
					InvalidateRect(hWnd, NULL, FALSE);
					break;
				case 'a':
				case 'A':
					PixmSettingsGet(pSettings, TAG_YPOSITION, 0, &lValue);
					PixmSettingsSet(pSettings, TAG_YPOSITION, 0, 0, lValue + 20);
					InvalidateRect(hWnd, NULL, FALSE);
					break;
				case 'l':
				case 'L':
					PixmSettingsGet(pSettings, TAG_XPOSITION, 0, &lValue);
					PixmSettingsSet(pSettings, TAG_XPOSITION, 0, 0, lValue - 20);
					InvalidateRect(hWnd, NULL, FALSE);
					break;
				case 'k':
				case 'K':
					PixmSettingsGet(pSettings, TAG_XPOSITION, 0, &lValue);
					PixmSettingsSet(pSettings, TAG_XPOSITION, 0, 0, lValue + 20);
					InvalidateRect(hWnd, NULL, FALSE);
					break;
				case 'h':
				case 'H':
					PixmSettingsSet(pSettings, TAG_XPOSITION, 0, 0, 0);
					PixmSettingsSet(pSettings, TAG_YPOSITION, 0, 0, 0);
					InvalidateRect(hWnd, NULL, FALSE);
					break;
#endif
			}
			break;

		case WM_PALETTECHANGED:
			/* Do nothing if we changed the palette */
			if (hWnd == (HWND)wParam) {
				return 0;
			} else {
				BOOL redraw = 0;
				HDC hDC = GetDC(hWnd);
				int ans = PixmPaletteChanged(hWnd, dwPagePalette, &redraw);
				if (redraw) {
					GetClientRect(hWnd, &rectClient);
					if (PixmRectPaint(hWnd, hDC,            /* DC, Window */
						mPage, pSettings, 0,              /* image data & settings */
						0, &rectClient,                   /* repaint area & posn */
						0, 0,                             /* no buffer passed */
						PIXM_PAINT_FAST|PIXM_PAINT_ACCEL| /* use Image Accel if avail */
						  PIXM_PAINT_NOMODIFY|dwPagePalette|PIXM_PAINT_FORCEPALETTE,
						0, prpcbRectCallBack))                            /* options */
						MessageBeep(0);                  /* just beep on error; avoid
														   infinite loop of repaints */
				}
				ReleaseDC(hWnd, hDC);
				return FALSE;
			}

		case WM_QUERYNEWPALETTE:
			{
				int ans;
				BOOL redraw = 0;
				HDC hDC = GetDC(hWnd);
				ans = PixmPaletteQueryNew(hWnd, dwPagePalette, &redraw);
				if (redraw) {
					GetClientRect(hWnd, &rectClient);
					if (PixmRectPaint(hWnd, hDC,            /* DC, Window */
						mPage, pSettings, 0,              /* image data & settings */
						0, &rectClient,                   /* repaint area & posn */
						0, 0,                             /* no buffer passed */
						PIXM_PAINT_FAST|PIXM_PAINT_ACCEL| /* use Image Accel if avail */
						  PIXM_PAINT_NOMODIFY|dwPagePalette|PIXM_PAINT_FORCEPALETTE,
						0, prpcbRectCallBack))                            /* options */
						MessageBeep(0);                  /* just beep on error; avoid
														   infinite loop of repaints */
				}
				ReleaseDC(hWnd, hDC);
				return ans;
			}

		case WM_PAINT:    /* code for the window's client area              */
			/* Obtain a handle to the device context                       */
			/* BeginPaint will sends WM_ERASEBKGND if appropriate          */
			memset(&ps, 0x00, sizeof(PAINTSTRUCT));
			hDC = BeginPaint(hWnd, &ps);

			if (mPage) {
#ifdef INCLUDE_PIXM_TEST
				if (bTestPattern)
					DrawTestPattern(
						bIsFulldrawn, hWnd, &ps, hDC, mPage, pSettings);
				else
#endif
				if (PixmRectPaint(hWnd, hDC,            /* DC, Window */
					mPage, pSettings, 0,              /* image data & settings */
					0, &ps.rcPaint,                   /* repaint area & posn */
					0, 0,                             /* no buffer passed */
					PIXM_PAINT_FAST|PIXM_PAINT_ACCEL| /* use Image Accel if avail */
					  PIXM_PAINT_NOMODIFY|dwPagePalette,
					0,prpcbRectCallBack))			  /* options */
					MessageBeep(0);                  /* just beep on error; avoid
													   infinite loop of repaints */
					dwPagePalette &= ~PIXM_PAINT_FORCEPALETTE;
#if defined INCLUDE_NOTES && !NOTECALLBACK
					if(NoteHndl[0]) {
						PixNoteScale(NoteHndl[0],hWnd,hDC,mPage,pSettings);
						PixNoteDraw(NoteHndl[0],hDC,0);
					}
#endif
			} else { /* clear screen to gray if no image */
					  /* we turn off auto-clear because it is not needed
						 for normal case */
				RECT Rect;
				GetClientRect(hWnd, &Rect);
				FillRect(hDC, &Rect, GetStockObject(GRAY_BRUSH));
			}
			if (hPixPan && PixPanIsVisible(hPixPan))
				PixPanUpdate(hPixPan);

			/* Inform Windows painting is complete                         */
			EndPaint(hWnd, &ps);
			break;       /*  End of WM_PAINT                               */

		case WM_VSCROLL:
			GetScrollRange(hWnd, SB_VERT, &i, &j);
			i = GetScrollPos(hWnd, SB_VERT);

			switch(GET_WM_VSCROLL_CODE(wParam,lParam)){
				case SB_LINEDOWN:
					i = min(j,i+25);
					break;

				case SB_LINEUP:
					i = max(0,i-25);
					break;

				case SB_THUMBPOSITION:
					i = GET_WM_VSCROLL_POS(wParam,lParam);
					break;

			case SB_THUMBTRACK:
				i = GET_WM_VSCROLL_POS(wParam, lParam);
			break;

				case SB_PAGEUP:
					i = max(0,i-100);
					break;

				case SB_PAGEDOWN:
					i = min(j,i+100);
					break;

				default:
					return FALSE;
			}

			SetScrollPos(hWnd, SB_VERT, i, TRUE);
#ifndef SLIDING_WINDOWS
			PixmSettingsSet(pSettings, TAG_YPOSITION, 0, 0, i);
#endif
			InvalidateRect(hWnd, NULL, FALSE);
			break;

		case WM_HSCROLL:
			GetScrollRange(hWnd, SB_HORZ, &i, &j);
			i = GetScrollPos(hWnd, SB_HORZ);

			switch(GET_WM_HSCROLL_CODE(wParam, lParam)) {
				case SB_LINEDOWN:
					i = min(j,i+25);
					break;

				case SB_LINEUP:
					i = max(0,i-25);
					break;

				case SB_THUMBPOSITION:
					i = GET_WM_HSCROLL_POS(wParam, lParam);
					break;

			case SB_THUMBTRACK:
				i = GET_WM_HSCROLL_POS(wParam, lParam);
			break;

				case SB_PAGEUP:
					i = max(0,i-100);
					break;

				case SB_PAGEDOWN:
					i = min(j,i+100);
					break;

				default:
					return FALSE;
			}

			SetScrollPos(hWnd, SB_HORZ, i, TRUE);
#ifndef SLIDING_WINDOWS
			PixmSettingsSet(pSettings, TAG_XPOSITION, 0, 0, i);
#endif
			InvalidateRect(hWnd, NULL, FALSE);
			break;

		case WM_CLOSE:  /* close the window                                */
			if (bBusy) {
				MessageBox(hWnd, "You must cancel scanning before quitting.",
					"Smplview", MB_OK);
				return 0L;
			}
			if (DoNoteCmd(hWnd, IDM_N_CLOSE, 0L))
				break;
			 /* Destroy child windows, modeless dialogs, then, this window */
			DestroyWindow(hWnd);
			ClosePanWindow();
			if (hWnd == hWndMain)
				PostQuitMessage(0);  /* Quit the application               */
			break;

		default:
			/* For any message for which you don't specifically provide a  */
			/* service routine, you should return the message to Windows   */
			/* for default message processing.                             */
			return DefWindowProc(hWnd, Message, wParam, lParam);
	}
	return 0L;
}     /* End of WndProc                                         */


/************************************************************************/
/*                                                                      */
/* Dialog Window Procedure                                              */
/*                                                                      */
/* This procedure is associated with the dialog box that is included in */
/* the function name of the procedure. It provides the service routines */
/* for the events (messages) that occur because the end user operates   */
/* one of the dialog box's buttons, entry fields, or controls.          */
/*                                                                      */
/************************************************************************/

BOOL PUBLIC_FUNC ABOUTMsgProc(HWND hWndDlg, UINT Message, WPARAM wParam, LPARAM lParam)
{

 switch(Message)
   {
	case WM_INITDIALOG:
		 cwCenter(hWndDlg, 0);
		 /* initialize working variables                                */
		 break; /* End of WM_INITDIALOG                                 */

	case WM_CLOSE:
		 /* Closing the Dialog behaves the same as Cancel               */
		 PostMessage(hWndDlg, WM_COMMAND,
			GET_WM_COMMAND_MPS(IDCANCEL, 0, 0));
		 break; /* End of WM_CLOSE                                      */

	case WM_COMMAND:
		 switch(GET_WM_COMMAND_ID(wParam, lParam))
		   {
			case IDOK:
				 EndDialog(hWndDlg, TRUE);
				 break;
			case IDCANCEL:
				 /* Ignore data values entered into the controls        */
				 /* and dismiss the dialog window returning FALSE       */
				 EndDialog(hWndDlg, FALSE);
				 break;
		   }
		 break;    /* End of WM_COMMAND                                 */

	default:
		return FALSE;
   }
 return TRUE;
} /* End of ABOUTMsgProc                                      */


/************************************************************************/
/*                                                                      */
/* nCwRegisterClasses Function                                          */
/*                                                                      */
/* The following function registers all the classes of all the windows  */
/* associated with this application. The function returns an error code */
/* if unsuccessful, otherwise it returns 0.                             */
/*                                                                      */
/************************************************************************/

int nCwRegisterClasses(void)
{
 WNDCLASS   wndclass;    /* struct to define a window class             */
 memset(&wndclass, 0x00, sizeof(WNDCLASS));


 /* load WNDCLASS with window's characteristics                         */
 wndclass.style = CS_HREDRAW | CS_VREDRAW | CS_BYTEALIGNCLIENT | CS_DBLCLKS;
 wndclass.lpfnWndProc = WndProc;
 /* Extra storage for Class and Window objects                          */
 wndclass.cbClsExtra = 0;
 wndclass.cbWndExtra = 0;
 wndclass.hInstance = hInst;
 wndclass.hIcon = LoadIcon(hInst, "SMPLVIEW");
#ifdef INCLUDE_NOTES
 wndclass.hCursor = 0;	/*We need to do our own cursor maintenance. 	*/
#else
 wndclass.hCursor = LoadCursor(NULL, IDC_ARROW);
#endif
 /* Create brush for erasing background                                 */
 wndclass.hbrBackground = (HBRUSH)0; /* (COLOR_WINDOW+1); */
 wndclass.lpszMenuName = szAppName;   /* Menu Name is App Name */
 wndclass.lpszClassName = szAppName; /* Class Name is App Name */
 if(!RegisterClass(&wndclass))
   return -1;


 return(0);
} /* End of nCwRegisterClasses                                          */

/************************************************************************/
/*  cwCenter Function                                                   */
/*                                                                      */
/*  centers a window based on the client area of its parent             */
/*                                                                      */
/************************************************************************/

void cwCenter(HWND hWnd, int top)
{
 POINT      pt;
 RECT       swp;
 RECT       rParent;
 int        iwidth;
 int        iheight;

 /* get the rectangles for the parent and the child                     */
 GetWindowRect(hWnd, &swp);
 GetClientRect(hWndMain, &rParent);

 /* calculate the height and width for MoveWindow                       */
 iwidth = swp.right - swp.left;
 iheight = swp.bottom - swp.top;

 /* find the center point and convert to screen coordinates             */
 pt.x = (rParent.right - rParent.left) / 2;
 pt.y = (rParent.bottom - rParent.top) / 2;
 ClientToScreen(hWndMain, &pt);

 /* calculate the new x, y starting point                               */
 pt.x = pt.x - (iwidth / 2);
 pt.y = pt.y - (iheight / 2);

 /* top will adjust the window position, up or down                     */
 if(top)
   pt.y = pt.y + top;

 /* move the window                                                     */
 MoveWindow(hWnd, pt.x, pt.y, iwidth, iheight, FALSE);
}

/************************************************************************/
/*  CwUnRegisterClasses Function                                        */
/*                                                                      */
/*  Deletes any refrences to windows resources created for this         */
/*  application, frees memory, deletes instance, handles and does       */
/*  clean up prior to exiting the window                                */
/*                                                                      */
/************************************************************************/

void CwUnRegisterClasses(void)
{
 WNDCLASS   wndclass;    /* struct to define a window class             */
 memset(&wndclass, 0x00, sizeof(WNDCLASS));

 UnregisterClass(szAppName, hInst);
}    /* End of CwUnRegisterClasses                                      */




